import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import persistence.AbstractSQLFactoryImpl;
import persistence.Utilities;

/*
 * Created on 19-feb-2004
 *
*/

/**
 * Generator voor implementor-aspecten uit persistentiebestanden.
 * Gebruik: java PersistenceImplementorGenerator implementor.xsl xml/classfile.xml xml aspects/imp
 * @author bram
 *
 * 19-feb-2004
 */
public class PersistenceImplementorGenerator {
	static Document document;
	static Collection persistentClasses;
	
	/**
	 * Transforms all classes out of xml-file to a java-file (aspect) by an xslt-stylesheet (implementor.xsl).
	 * @param argv
	 */
	public static void main (String argv [])
	{		
		if (argv.length != 4) {
			System.err.println ("Usage: java PersistenceImplementorGenerator stylesheet xmlfile relative srcDir without \'./\' destDir idem");
			System.exit (1);
		}
		
		persistentClasses=new HashSet();
		
		//1 uit classfile paden naar persistency files halen en namen aspecten
		ArrayList persistencyFiles=getPersistencyFiles(argv[1],System.getProperty("user.dir")+System.getProperty("file.separator")+argv[2]);
		System.out.println(persistencyFiles.toString());
		AbstractSQLFactoryImpl.setAllClassesToPersist(persistentClasses);
		
		//2 die files transformeren naar aspecten met namen met _'s
		DocumentBuilderFactory factory =
			DocumentBuilderFactory.newInstance();
		Iterator it=persistencyFiles.iterator();
		while(it.hasNext()){
			Vector tmp=(Vector)it.next();
			String xmlFilePath=(String)tmp.get(0);
			String aspectPathName=(String)tmp.get(1);
			String aspectFileName=(String)tmp.get(2);
		try {
			File datafile   = new File(xmlFilePath);
			File stylesheet = new File(argv[0]);
			String destDir=(argv[3].endsWith(System.getProperty("file.separator")))?argv[3]:argv[3]+System.getProperty("file.separator");
			File curDir=new File(destDir+aspectPathName);
			curDir.mkdirs();						
			File resFile=new File(destDir+aspectPathName+aspectFileName);
 
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.parse(datafile);
 
			// Use a Transformer for output
			TransformerFactory tFactory =
				TransformerFactory.newInstance();
			StreamSource stylesource = new StreamSource(stylesheet);
			Transformer transformer = tFactory.newTransformer(stylesource);
 
			DOMSource source = new DOMSource(document);
			StreamResult result = new StreamResult(resFile);
			transformer.transform(source, result);
           
		} catch (TransformerConfigurationException tce) {
		   // Error generated by the parser
		   tce.printStackTrace();
      
		} catch (TransformerException te) {
		   // Error generated by the parser		   
		   te.printStackTrace();
           
		 } catch (SAXException sxe) {
		   // Error generated by this application
		   // (or a parser-initialization error)
		   sxe.printStackTrace();

		} catch (ParserConfigurationException pce) {
			// Parser with specified options can't be built
			pce.printStackTrace();

		} catch (IOException ioe) {
		   // I/O error
		   ioe.printStackTrace();
		}	
 		}
	}
	
	/**
	 * Opvragen van paden naar alle persistentie-files.
	 * @param xmlFile
	 * @param dir
	 * @return
	 */
	private static ArrayList getPersistencyFiles(String xmlFile,String dir){

		class PathExtractor extends DefaultHandler{
			private ArrayList res;			
			private String dir;
					
			public PathExtractor(ArrayList res,String dir){
				this.res=res;
				this.dir=dir;
			}
			
			public void startElement(String namespaceURI,
										 String sName, // simple name (localName)
										 String qName, // qualified name
										 Attributes attrs)
				throws SAXException
				{				
					if(qName.equals("classUnit")){
						String s=attrs.getValue("name");						
						System.out.println(s);
						persistentClasses.add(s);//alle persistent container classes toevoegen!
									
						Vector vec=new Vector();
						vec.add(preparePackageDir(dir,s));
						String separator=System.getProperties().getProperty("file.separator");
						vec.add(s.substring(0,s.lastIndexOf(".")+1).replaceAll("\\.",separator));//OK: als in default package, dan niets!							
						vec.add(Utilities.prepareAspectName(s)+".java");
						res.add(vec);
					}					
				}
			
			public void endElement(String namespaceURI,
									   String sName, // simple name
									   String qName  // qualified name
									  )
				throws SAXException
				{}
				
			public void characters(char buf[], int offset, int len)
				throws SAXException
				{}
		
			/**
			 * Construct full path to persistency files.
			 */
			private String preparePackageDir(String dirPath,String fullClassName){				
				String separator=System.getProperties().getProperty("file.separator");
				String res=dirPath;
				if(!dirPath.endsWith(separator)){
					res+=separator;
				}
				String xmlName="pdl-"+fullClassName.substring(fullClassName.lastIndexOf('.')+1)+".xml";				
				res+=fullClassName.substring(0,fullClassName.lastIndexOf(".")+1).replaceAll("\\.",separator)+xmlName;//remove classname and replace '.' by separator

				return res;
			}
						
		
		}		
		
		ArrayList res=new ArrayList();

		DefaultHandler handler = new PathExtractor(res,dir);

		// Use the default (non-validating) parser
		SAXParserFactory factory = SAXParserFactory.newInstance();
		try {
			 // Parse the input
			 SAXParser saxParser = factory.newSAXParser();
			 saxParser.parse( new File(xmlFile), handler );

		} catch (Throwable t) {
			 t.printStackTrace();
		}
		return res;
	}
	

}
