import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/*
 * Created on 12-okt-2003
 *
*/

/**
 * Generator van introducer-aspect uit klassenlijst.
 * Gebruik: java PersistenceIntroducerGenerator introducer.xsl xml/classfile.xml aspects/PersistenceIntroducer.java
 * @author bram
 *
 * 12-okt-2003
 */
public class PersistenceIntroducerGenerator {
	static Document document;
	
	/**
	 * Transforms an xml-file to a java-file (aspect) by a xslt-stylesheet (introducer.xsl).
	 * @param argv
	 */
	public static void main (String argv [])
	{
		if (argv.length != 3) {
			System.err.println ("Usage: java PersistenceIntroducerGenerator stylesheet xmlfile resfile");
			System.exit (1);
		}

		DocumentBuilderFactory factory =
			DocumentBuilderFactory.newInstance();
 
		try {
			File stylesheet = new File(argv[0]);
			File datafile   = new File(argv[1]);
			File resFile=new File(argv[2]);
 
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.parse(datafile);
 
			// Use a Transformer for output
			TransformerFactory tFactory =
				TransformerFactory.newInstance();
			StreamSource stylesource = new StreamSource(stylesheet);
			Transformer transformer = tFactory.newTransformer(stylesource);
 
			DOMSource source = new DOMSource(document);
			StreamResult result = new StreamResult(resFile);
			transformer.transform(source, result);
           
		} catch (TransformerConfigurationException tce) {
		   // Error generated by the parser
		   tce.printStackTrace();
      
		} catch (TransformerException te) {
		   // Error generated by the parser		   
		   te.printStackTrace();
           
		 } catch (SAXException sxe) {
		   // Error generated by this application
		   // (or a parser-initialization error)
		   sxe.printStackTrace();

		} catch (ParserConfigurationException pce) {
			// Parser with specified options can't be built
			pce.printStackTrace();

		} catch (IOException ioe) {
		   // I/O error
		   ioe.printStackTrace();
		}	
	}
}
