/*
 * Created on 5-mrt-2004
 *
*/
package aspects;

import java.util.Vector;

import persistence.cache.Cache;
import persistence.cache.SimpleCache;
import persistence.cache.exceptions.AlreadyInCacheException;
import persistence.cache.exceptions.NotInCacheException;
import persistence.exceptions.DeleteException;
import persistence.exceptions.UpdateException;
import persistence.exceptions.WriteException;
import persistence.type.ObjectID;
import persistence.type.Persistent;

/**
 * Primitieve cache.
 * @author bram
 *
 * 5-mrt-2004
 */
public abstract aspect CacheAspect {
	declare precedence: CacheAspect,ConnectionAspect;
	
	protected static Cache cache=new SimpleCache();
	
	private boolean persistence.type.Persistent._dirty=false;
	
	public boolean persistence.type.Persistent.is_dirty(){
		return _dirty;
	}
	
	public void persistence.type.Persistent.set_dirty(boolean _dirty){
		this._dirty=_dirty;
	}
	
	abstract pointcut setters();
	
	after(Persistent obj):setters()&&this(obj){
		obj.set_dirty(true);
	}
	
	/**
	 * Altijd de deep-versie: insert!
	 * Enkel bij read kan writeShallow... gebruikt worden!
	 */
	void around(Persistent obj) throws WriteException:execution(public void Persistent+.write() throws WriteException)&&this(obj){		
		if(!cache.checkForDeepOID(obj.getOID())){ //nog niet eerder in cache (dus niet eerder opgeslagen)
			proceed(obj);//eerst wegschrijven en indien dit gelukt is in cache			
			try{
				System.out.println("Schrijven van "+obj.getOID()+" in cache ... ");
				cache.writeDeepToCache(obj.getOID(),obj);
			}catch(AlreadyInCacheException e){
				System.out.println("OK<-->zou niet mogen!");
				System.err.print(e.getMessage());
			}			
		}		
	}
  	
	//around: lees eerst uit cache, indien niet aanwezig: proceed en in cache steken afhankelijk van
	//deep=true of niet
	Object around(ObjectID OID,boolean deep):execution(public static Object Persistent.QueryManager.read(ObjectID,boolean))&&args(OID,deep){
		System.out.println("Zoeken in cache ...");
		Object obj=null;
		
		try{
			if(deep){
				obj=cache.readDeep(OID);
			}else{
				obj=cache.readShallow(OID);
			}
			System.out.println("Aanwezig in cache!");
		}catch(NotInCacheException ex){
			System.out.println("Niet in cache aanwezig: zal het uit DB halen!");
			obj=proceed(OID,deep);
			try{
			if(deep){
				cache.writeDeepToCache(OID,obj);
			}else{
				cache.writeShallowToCache(OID,obj);
			}
			}catch(AlreadyInCacheException e){
				//OK
			}
		}
		
		return obj;
	}
  	
  	//niet aan cache komen: alles wijst toch naar uniek object in cache
  	//<-->welke versie gepdated? niet belangrijk, toch juiste die dirty-af is: verantwoordleijkheid applicatie om full versie op te slaan
  	//en daarbij: enkel problemen bij compositie omdat daar de composites ook opgeslagen worden! bij rest enkel OID belangrijk
	void around(Persistent obj) throws UpdateException:execution(public void Persistent+.update() throws UpdateException)&&this(obj){		
		if(obj.is_dirty()){
			proceed(obj);
			obj.set_dirty(false);
		}		
	}
  	
  	//haal het uit cache na delete, want blijft toch nog transient bestaan
	//void around(Persistent obj) throws DeleteException:execution(public void Persistent+.delete() throws DeleteException)&&this(obj){		
	after(Persistent obj) throws DeleteException:execution(public void Persistent+.delete() throws DeleteException)&&this(obj){
		//proceed(obj);
		cache.remove(obj.getOID());
	}
	//in gesofisticeerder CacheAspect :
	//around-advice checkt eerst of veranderen mag, indien ja wordt methode op allen in queue en in cache uitgevoerd
	//finalize verwijdert object uit queue van cache
}
