/*
 * Created on 5-apr-2004
 *
*/
package aspects;

import java.sql.Connection;
import java.sql.SQLException;

import persistence.transaction.Transaction;
import persistence.transaction.TransactionImpl;
import persistence.type.ConnectionFactory;

/**
 * Supertransactie-support.
 * @author bram
 *
 * 5-apr-2004
 */
public aspect SuperTransactionAspect {
	protected ConnectionFactory fac=new ConnectionFactory();
	
	declare precedence:ConnectionAspect,SuperTransactionAspect,TransactionAspect;
	//aan de ene kant heeft TransactionAspect het al gedeeltelijk vastgelegd + 't is core
	//<-->requirements opgeven
	
	private Connection persistence.transaction.Transaction.currentConnection=null;
	
	public Connection persistence.transaction.Transaction.getCurrentConnection(){
		return this.currentConnection;
	}
	
	public void persistence.transaction.Transaction.setCurrentConnection(Connection currentConnection){
		this.currentConnection=currentConnection;
	}
	
	private boolean persistence.transaction.Transaction.active=false;//started?
	
	public boolean persistence.transaction.Transaction.checkCurrentConnection(){
		try{
			return (this.currentConnection!=null)&&(!this.currentConnection.isClosed())&&(!this.currentConnection.getAutoCommit());
		}catch(Exception e){}
		return false;
	}
	
	boolean around(persistence.transaction.Transaction trans):execution(public boolean persistence.transaction.Transaction+.isActive())&&this(trans){
		return trans.active;
	} 
	
	public void persistence.transaction.Transaction.setActive(boolean active){
		this.active=active;
	}
	
	/**
	 * getConnection()
	 */
	Connection around(Transaction trans):execution(public Connection persistence.type.ConnectionFactory.getConnection())&&cflow(PersistenceIntroducer.tpersistentExecution(trans))&&!within(aspects.SuperTransactionAspect){
		if(trans.isActive()&&trans.checkCurrentConnection()){ 
			System.out.println("Using connection contained in "+trans.getTransactionID());
			return trans.getCurrentConnection();
		}else{
			if(trans.isActive()){ //connection gone
				System.err.println("Connection is gone: exception in inner transaction has occurred, but I wasn't rolled back!!");
				try{
					trans.getCurrentConnection().rollback();
				}catch(Exception e){
					//did my best
				}
				return null;
			}else{ //laat gewone TransactionAspect zijn werk doen: zet zelf autoCommit op false
				System.out.println("Transaction "+trans.getTransactionID()+" not active.");
				Connection tmp=proceed(trans);
				return tmp;
			}	
		}
	}
	
	/**
	 * closeConnection() 
	 */
	void around(persistence.transaction.Transaction trans):execution(public void persistence.type.ConnectionFactory.closeConnection(java.sql.Connection))&&cflow(PersistenceIntroducer.tpersistentExecution(trans))&&!within(aspects.SuperTransactionAspect){//ConnectionAspect.connectionClosing(){
		if(trans.isActive()){
			System.out.println("Transaction "+trans.getTransactionID()+" still active. Not closing connection!");
			return; 
			}
		else{ //laat gewone TransactionAspect zijn werk doen
			System.out.println("Transaction "+trans.getTransactionID()+" not active. Passing to regular TransactionAspect!");
			proceed(trans);
			return; 	
		}
	}
	
	/**
	 * start() 
	 */
	void around(persistence.transaction.Transaction trans):execution(public void persistence.transaction.Transaction+.start())&&this(trans){
		//actief maken en connectie opvragen
		if(trans.isActive()){
			System.err.println("Transaction already started!");
			return;
		}
		trans.setActive(true);
		System.out.println("Start of transaction "+trans.getTransactionID()+".");
		Connection tmp=fac.getConnection();
		try{
			tmp.setAutoCommit(false);
		}catch(SQLException ex){
			System.out.println("Mislukte setAutoCommit(false) voor "+trans.getTransactionID()+".");
		}
		trans.setCurrentConnection(tmp);
	}
	
	
	/**
	 * commit()
	 */
	void around(persistence.transaction.Transaction trans):execution(public void persistence.transaction.Transaction+.commit())&&this(trans){
		if(trans.isActive()&&trans.checkCurrentConnection()){
			System.out.println("Committing transaction "+trans.getTransactionID()+"!");
			try{
				trans.getCurrentConnection().commit();
			}catch(SQLException ex){
				System.err.println("Committing doesn't work: trouble!!");
			}finally{ //zie closeConnection()
				try{
					fac.closeConnection(trans.getCurrentConnection());
				}catch(Exception ex){
				}
			}
			trans.setActive(false);		
		}else{
			if(trans.isActive()){
				System.err.println("Connection is gone: trouble!! Disactivating transaction "+trans.getTransactionID()+"!");
				trans.setActive(false);
			}else{ 
				System.out.println("Transaction "+trans.getTransactionID()+" not active. Unnecessary commit!");
			}
		}
		return;
	}
	
	/**
	 * rollback()
	 */
	void around(persistence.transaction.Transaction trans):execution(public void persistence.transaction.Transaction+.rollback())&&this(trans){
		if(trans.isActive()){ //rollback necessary
			System.out.println("Rolling back "+trans.getTransactionID()+"...");
			try{
				trans.getCurrentConnection().rollback();
			}catch(SQLException ex){
				System.err.println("Rolling back doesn't work: trouble!!");
			}finally{
				try{
					fac.closeConnection(trans.getCurrentConnection());
				}catch(Exception ex){
				}
			}
			trans.setActive(false);
		}
	}

}
