import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.DocErrorReporter;
import com.sun.javadoc.Doclet;
import com.sun.javadoc.RootDoc;

import doclet.ClassFileGenerator;
import doclet.TagException;
import doclet.XMLDoclet;

/*
 * Created on 11-feb-2004
 *
*/

/**
 * Doclet voor PDL-tags.
 * @author bram
 *
 * 11-feb-2004
 */
public class MainDoclet extends Doclet {
	private static Collection visitedClasses = new HashSet();
	//nodig voor filteren van niet-classUnit's (zullen vanzelf wel bekeken worden)
	private static Map containerMap = new HashMap();
	//container class name naar Collection van inners (zonder container)
	private static String xmlDirPath;
	private static XMLDoclet gen = new XMLDoclet();

	public static boolean start(RootDoc root) {
		xmlDirPath = readOptions(root.options());

		ClassDoc[] classes = root.classes();
		for (int i = 0; i < classes.length; i++) {
			String fullClassName = classes[i].qualifiedName();
			if (!visitedClasses.contains(fullClassName)) {
				Collection inners = filterInnerClasses(classes[i]);

				if (!isInterestingClass(classes[i]))
					continue;
				visitedClasses.add(fullClassName);
				containerMap.put(fullClassName, inners);
				System.out.println("Container bekeken: " + fullClassName);
				String res =
					"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
						+ "<classUnit name=\""
						+ fullClassName
						+ "\">";
				try {
					res += processClass(classes[i]);
				} catch (TagException e1) {
					System.out.println(
						"Error while processing " + fullClassName + "!");
					e1.printStackTrace();
					continue;
				}
				res += "</classUnit>";

				String relPath = preparePackageDir(xmlDirPath, classes[i]);
				String fileName = "pdl-" + classes[i].typeName() + ".xml";
				try {
					File curDir = new File(xmlDirPath + relPath);
					curDir.mkdirs();
					PrintWriter writer =
						new PrintWriter(
							new BufferedWriter(
								new FileWriter(
									xmlDirPath + relPath + fileName)));
					writer.print(res);
					writer.close();
				} catch (IOException e) {
					System.err.println(
						"Problem whilst writing file "
							+ xmlDirPath
							+ relPath
							+ fileName
							+ "!");
					e.printStackTrace();
				}
			}
		}
		try {

			ClassFileGenerator claGen = new ClassFileGenerator();
			claGen.setContainerMap(containerMap);
			PrintWriter writer =
				new PrintWriter(
					new BufferedWriter(
						new FileWriter(xmlDirPath + "classfile.xml")));
			writer.print(claGen.generate(null));
			writer.close();
		} catch (IOException e) {
			System.err.println(
				"Problem whilst writing file "
					+ xmlDirPath
					+ "classfile.xml"
					+ "!");
			e.printStackTrace();
		}
		return true;
	}

	/**
	 * Only normal classes (no interface/exception/error) marked &at;persistent and having a &at;pversion 
	 * @param claDo
	 * @return
	 */
	private static boolean isInterestingClass(ClassDoc claDo) {
		boolean res = true;
		if (!(claDo.isOrdinaryClass() || claDo.isInterface()))
			res = false;
		//enkel gewone klassen en interfaces											
		if (claDo.tags("@persistent").length == 0)
			res = false;
		//enkel @persistent klassen												
		//interfaces hebben enkel @persistent-tag!
		if (!claDo.isInterface()
			&& ((claDo.tags("@pversion").length == 0)
				|| (!(claDo.tags("@pversion")[0])
					.text()
					.matches("(?s).*major.*")))) {
			System.err.println("@pversion major-tag required!");
			res = false;
		}

		return res;
	}

	/**
	 * Prepare package name to a path name.
	 * @param dirPath
	 * @param claDo
	 * @return
	 */
	private static String preparePackageDir(String dirPath, ClassDoc claDo) {
		String separator = System.getProperties().getProperty("file.separator");
		String res = "";
		if (!dirPath.endsWith(separator)) {
			res += separator;
		}
		res
			+= claDo
				.qualifiedName()
				.substring(0, claDo.qualifiedName().lastIndexOf(".") + 1)
				.replaceAll("\\.", separator);
		//remove classname and replace '.' by separator
		return res;
	}

	/**
	 * 
	 * @param claDo
	 * @return XML-String
	 */
	private static String processClass(ClassDoc claDo) throws TagException {
		System.out.println("processing " + claDo.qualifiedName());
		ClassDoc[] inner = claDo.innerClasses();
		System.out.println("number of inners: " + inner.length);
		String res = "";
		res += gen.generate(claDo);

		if (inner.length != 0) { //at least one inner class

			for (int i = 0; i < inner.length; i++) {
				res += processClass(inner[i]);
			}
		}
		return res;
	}

	/**
	 * Markeer inner classes als gezien, want zij worden beschreven in persistentiefile van hun container
	 * @param claDo
	 */
	private static Collection filterInnerClasses(ClassDoc claDo) {
		Collection res = new ArrayList();
		ClassDoc[] inner = claDo.innerClasses();
		for (int i = 0; i < inner.length; i++) {
			if (isInterestingClass(claDo)) {
				visitedClasses.add(inner[i].qualifiedName());
				res.add(inner[i].qualifiedName());
			}
			res.addAll(filterInnerClasses(inner[i]));
		}
		return res;
	}

	/**
	 * Geef waarde van optie -dirpath
	 * @param options
	 * @return
	 */
	private static String readOptions(String[][] options) {
		String dirPath = null;
		for (int i = 0; i < options.length; i++) {
			String[] opt = options[i];
			if (opt[0].equals("-dirpath")) {
				dirPath = opt[1];
			}
		}
		return dirPath;
	}

	/**
	 * Geef lengte van extra optie -dirpath
	 * @see com.sun.javadoc.Doclet#optionLength(java.lang.String)
	 */
	public static int optionLength(String option) {
		if (option.equals("-dirpath")) {
			return 2;
		}
		return 0;
	}

	/**
	 * Controleer extra optie -dirpath
	 * @see com.sun.javadoc.Doclet#validOptions(java.lang.String[][], com.sun.javadoc.DocErrorReporter)
	 */
	public static boolean validOptions(
		String options[][],
		DocErrorReporter reporter) {
		boolean foundTagOption = false;
		for (int i = 0; i < options.length; i++) {
			String[] opt = options[i];
			if (opt[0].equals("-dirpath")) {
				if (foundTagOption) {
					reporter.printError("Only one -dirpath option allowed.");
					return false;
				} else {
					foundTagOption = true;
				}
			}
		}
		if (!foundTagOption) {
			reporter.printError(
				"Usage: javadoc -dirpath mypath -doclet MainDoclet ...");
		}
		return foundTagOption;
	}

}
