/*
 * Created on 15-okt-2003
 *
*/
package persistence;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.w3c.dom.Node;

import persistence.exceptions.ClassNotRegisteredException;
import persistence.exceptions.CompTypeRequiredException;
import persistence.exceptions.MissingContentTagException;
import persistence.script.DBScript;
import persistence.table.TableComponent;

/**
 * @author bram
 *
 * 15-okt-2003
 */
public interface AbstractSQLFactory {
	/**
	 * Create String with all needed statements for insertion of object whose persistence properties are described
	 * in NodeList. 
	 * @param thisObjectName name of current Object
	 * @return
	 */
	public String getInsertStatement();

	/**
		 * Create String with all needed statements for update of object whose persistence properties are described
		 * in NodeList. 
		 * @param thisObjectName name of current Object
		 * @return
		 */
	public String getUpdateStatement();

	/**
		 * Create String with all needed statements for deletion of object whose persistence properties are described
		 * in NodeList. 
		 * @param thisObjectName name of current Object
		 * @return
		 */
	public String getDeleteStatement();

	/**
		 * Create String with all needed statements for creation of tables for objects of the class whose persistence properties are described
		 * in NodeList.
		 * @param dialectVariableName 		 
		 * @return
		 */
	public String getCreateStatement(String dialectVariableName);

	/**
		 * Create String with all needed statements for dropping of tables for objects of the class whose persistence properties are described
		 * in NodeList.
		 * @param dialectVariableName 		 
		 * @return
		 */
	public String getDropStatement(String dialectVariableName);

	/**
		 * Sets a NodeList of &lt;persistentClass&gt;-elements as the current one and extraction of useful info happens.
		 * @param node
		 * @param fullPrincipalClassName
		 */
	public void setCurrentNodeList(Node node, String fullPrincipalClassName)
		throws CompTypeRequiredException, MissingContentTagException;

	/**
	 * Adds this class's create/drop code to script.	 
	 */
	public void addToScript();

	/**
	 * Create String with all needed statements for reading object whose persistence properties are described
	 * in NodeList.
	 * @param nameOfOID
	 * @return
	 */
	public String getReadStatement();

	public String getQueryStatements();

	public static Registry registry = new Registry();

	/**
	 * Experimentele klasse om TableComponents desnoods zelf op te slaan (eerst nog anno-
	 * teren).
	 * @author bram
	 *
	 * 26-april-2004
	 */
	public static class Registry {
		//		Map van typeName naar map van TreeInfo naar TableComponent
		private Map parsedPersistenceTrees = new HashMap();

		/**
		 * Access Map from TreeInfo's to TableComponent corresponding to persistent class with fullPersistentClassName.
		 * @param fullPersistentClassName
		 * @return
		 * @throws ClassNotRegisteredException
		 */
		public Map getPersistenceTree(String fullPersistentClassName)
			throws ClassNotRegisteredException {
			if (parsedPersistenceTrees.containsKey(fullPersistentClassName)) {
				return (Map) parsedPersistenceTrees.get(
					fullPersistentClassName);
			} else {
				throw new ClassNotRegisteredException();
			}
		}

		public ArrayList getLatestPersistenceTrees() {
			ArrayList res = new ArrayList();
			Iterator it = parsedPersistenceTrees.entrySet().iterator();
			while (it.hasNext()) {
				Map tmp = (Map) ((Map.Entry) it.next()).getValue();
				Iterator ot = tmp.entrySet().iterator();
				Map.Entry tmp2 = null;
				while (ot.hasNext())
					tmp2 = (Map.Entry) ot.next();
				//itereer naar laatste, dit is recentste
				res.add(tmp2.getValue());
			}
			return res;
		}

		public void addPersistenceTree(
			String fullPersistentClassName,
			TableComponent tComponent) {
			Map tree = null;
			try {
				tree = getPersistenceTree(fullPersistentClassName);
				removePersistenceTree(fullPersistentClassName);
			} catch (ClassNotRegisteredException e) {
				tree = new HashMap();
			}
			TreeInfo info =
				new TreeInfo(
					fullPersistentClassName,
					Integer.parseInt(tComponent.getProperty("major")),
					Integer.parseInt(tComponent.getProperty("minor")));
			tree.put(info, tComponent);

			parsedPersistenceTrees.put(fullPersistentClassName, tree);
		}

		public void removePersistenceTree(String fullPersistentClassName) {
			//alle versies verwijderd
			parsedPersistenceTrees.remove(fullPersistentClassName);
		}
	}

	/**
	 * Nog experimentelere klasse die als key zou kunnen dienen.
	 * @author bram
	 *
	 * 26-april-2004
	 */
	public static class TreeInfo implements Comparable {
		private String typeName;
		private int major;
		private int minor;

		public TreeInfo(String typeName, int major, int minor) {
			this.typeName = typeName;
			this.major = major;
			this.minor = minor;
		}

		/**
		 * @return
		 */
		public int getMajor() {
			return major;
		}

		/**
		 * @return
		 */
		public int getMinor() {
			return minor;
		}

		/**
		 * @return
		 */
		public String getTypeName() {
			return typeName;
		}

		/**
		 * @param i
		 */
		public void setMajor(int i) {
			major = i;
		}

		/**
		 * @param i
		 */
		public void setMinor(int i) {
			minor = i;
		}

		/**
		 * @param string
		 */
		public void setTypeName(String string) {
			typeName = string;
		}

		/**
		 * @see java.lang.Object#equals(java.lang.Object)
		 */
		public boolean equals(Object obj) {
			TreeInfo other = (TreeInfo) obj;
			return typeName.equals(other.typeName)
				&& (minor == other.minor)
				&& (major == other.major);
		}

		/**
		 * @see java.lang.Comparable#compareTo(java.lang.Object)
		 */
		public int compareTo(Object o) {
			TreeInfo other = (TreeInfo) o;
			int typeCompare = other.typeName.compareTo(typeName);
			if (typeCompare == 0) { //zelfde type
				if (other.major == major) { //zelfde major
					return minor - other.minor; //check minor
				} else { //verschillende major
					return major - other.major;
				}
			} else { //verschillend type
				return typeCompare;
			}
		}

	}

}
