/*
 * Created on 15-okt-2003
 *
*/
package persistence;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

/**
 * Encapsulates necessary info for a database connection.  
 * @author bram
 *
 * 15-okt-2003
 */
public class DatabaseConnection {
	protected Connection conn;
	protected String driverClassName;
	protected String connectionURL;
	private String userName;
	private String passWord;

	/**
	 * Create a database connection.
	 * @param driverClassName
	 * @param connectionURL
	 * @param userName
	 * @param passWord
	 */
	public DatabaseConnection(
		String driverClassName,
		String connectionURL,
		String userName,
		String passWord)
		throws ClassNotFoundException, SQLException {
		this.connectionURL = connectionURL;
		this.driverClassName = driverClassName;
		this.userName = userName;
		this.passWord = passWord;
		Class.forName(this.driverClassName);
		this.conn =
			DriverManager.getConnection(
				this.connectionURL,
				this.userName,
				this.passWord);

	}

	/**
	 * Create a database connection.
	 * @param props
	 */
	public DatabaseConnection(Properties props)
		throws ClassNotFoundException, SQLException {
		this.connectionURL = props.getProperty("db.URL");
		this.driverClassName = props.getProperty("db.driverClassName");
		this.userName = props.getProperty("db.user");
		this.passWord = props.getProperty("db.passWord");
		Class.forName(this.driverClassName);
		this.conn =
			DriverManager.getConnection(
				this.connectionURL,
				this.userName,
				this.passWord);
	}

	/**
	 * @return
	 */
	public Connection getConn() {
		return conn;
	}

	/**
	 * @return
	 */
	public String getConnectionURL() {
		return connectionURL;
	}

	/**
	 * @return
	 */
	public String getDriverClassName() {
		return driverClassName;
	}

}
