/*
 * Created on 23-okt-2003
 *
*/
package persistence;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import persistence.exceptions.ClassNotRegisteredException;
import persistence.exceptions.CompTypeRequiredException;
import persistence.exceptions.MissingContentTagException;
import persistence.table.*;

/**
 * Parser van persistentClass-tags uit persistentiebestand naar TableComponent-structuur.
 * @author bram
 *
 * 23-okt-2003
 */
public class PDLParser {
	private Node involvedNode;
	private String fullPrincipalClassName;
	private TableComponent currentRoot;
	private static Collection allClassesToPersist;

	private String setMethodName;
	private String getMethodName;
	private String attributeName;
	private String isAccessor;
	private boolean addArrayString = false;
	//compType van Map/Collection moet "[]"s bevatten
	private final int maxSizeTableName = 64;

	static {
		allClassesToPersist = new Vector();
		//by default the parents are NOT to be persisted
	}

	/*
		 * @author bram
		 *
		 * 21-okt-2003
		 */
	/**
	 * Contains queue of Maps which contains mappings:
	 * array -> Integer (default 0, dus geen array) 
	 * compType -> String (verplicht != "")
	 * size -> Integer (default 0)
	 * impClass -> String (default "")
	 * compTypeKey -> String (default "")
	 * sizeKey -> Integer (default 0)
	 * impClassKey -> String (default "")
	 * contained -> String (default "false")	 
	 * containedKey -> String (default "false")
	 *  
	 * radt -> String
	 * @author bram
	 *
	 * 23-okt-2003
	 */
	private class ContentTagList {
		private ArrayList tags;

		public ContentTagList(NodeList contentNodes)
			throws CompTypeRequiredException {
			tags = new ArrayList();
			List temp = filterWhiteSpaceNodes(contentNodes);

			Iterator it = temp.iterator();
			while (it.hasNext()) {
				addContentTag((Node) it.next());
			}
		}

		public ContentTagList() {
			tags = new ArrayList();
		}

		public int getSize() {
			return tags.size();
		}

		public Map pullContentTag() throws IndexOutOfBoundsException {
			return (Map) tags.remove(0);
		}

		public Map peekContentTag() throws IndexOutOfBoundsException {
			return (Map) tags.get(0);
		}

		public void addContentTag(Node contentTag)
			throws CompTypeRequiredException {
			NamedNodeMap attrMap = contentTag.getAttributes();
			Map temp = new HashMap();

			Node tempNode = attrMap.getNamedItem("compType"); //compType
			if (tempNode != null)
				temp.put("compType", tempNode.getNodeValue());
			else
				throw new CompTypeRequiredException();
			tempNode = attrMap.getNamedItem("size"); //size
			if (tempNode != null)
				temp.put("size", new Integer(tempNode.getNodeValue()));
			else
				temp.put("size", new Integer(0));
			tempNode = attrMap.getNamedItem("impClass");
			//impClass				
			if (tempNode != null)
				temp.put("impClass", tempNode.getNodeValue());
			else
				temp.put("impClass", "");
			tempNode = attrMap.getNamedItem("contained"); //contained
			if (tempNode != null)
				temp.put("contained", tempNode.getNodeValue());
			else
				temp.put("contained", "false");
			tempNode = attrMap.getNamedItem("containedKey"); //containedKey
			if (tempNode != null)
				temp.put("containedKey", tempNode.getNodeValue());
			else
				temp.put("containedKey", "false");
			tempNode = attrMap.getNamedItem("compTypeKey"); //compTypeKey
			if (tempNode != null)
				temp.put("compTypeKey", tempNode.getNodeValue());
			else
				temp.put("compTypeKey", "");
			tempNode = attrMap.getNamedItem("sizeKey"); //sizeKey
			if (tempNode != null)
				temp.put("sizeKey", new Integer(tempNode.getNodeValue()));
			else
				temp.put("sizeKey", new Integer(0));
			tempNode = attrMap.getNamedItem("impClassKey");
			//impClassKey				
			if (tempNode != null)
				temp.put("impClassKey", tempNode.getNodeValue());
			else
				temp.put("impClassKey", "");
			tempNode = attrMap.getNamedItem("array"); //size
			if (tempNode != null)
				temp.put("array", new Integer(tempNode.getNodeValue()));
			else
				temp.put("array", new Integer(0));

			tags.add(temp);

		}

	}

	public PDLParser(Node involvedNode, String fullPrincipalClassName) {
		this.involvedNode = involvedNode;
		this.fullPrincipalClassName = fullPrincipalClassName;
	}

	/**
	 * Recursive depth-first creation of TableComponent-treecomponents of currentRoot.	 	
	 */
	public TableComponent parse()
		throws CompTypeRequiredException, MissingContentTagException {
		boolean tableNameProvided = false;
		Node node = null;
		NodeList children = involvedNode.getChildNodes();
		List nonWhiteSpaceNodes = filterWhiteSpaceNodes(children);

		System.out.println(
			involvedNode.getAttributes().getNamedItem("name").getNodeValue());

		NamedNodeMap attrMap = involvedNode.getAttributes();
		//attributes of persistentClass-element
		String currentClassName = attrMap.getNamedItem("name").getNodeValue();

		String currentExtendedClassName =
			attrMap.getNamedItem("extends").getNodeValue();

		Iterator it = nonWhiteSpaceNodes.iterator();

		Node childOne = (Node) it.next();
		NamedNodeMap dbMap = childOne.getAttributes();

		String database = dbMap.getNamedItem("database").getNodeValue();
		String table = dbMap.getNamedItem("table").getNodeValue();
		if ((table != null) && (!table.equals("")))
			tableNameProvided = true;

		Node childTwo = (Node) it.next();
		NamedNodeMap versionMap = childTwo.getAttributes();
		int major =
			(new Integer(versionMap.getNamedItem("major").getNodeValue()))
				.intValue();

		int minor =
			(new Integer(versionMap.getNamedItem("minor").getNodeValue()))
				.intValue();

		String currentTable = "";
		currentTable =
			constructTableName(currentClassName, table, major, minor);
		currentRoot = new Table();
		currentRoot.addProperty("root", "true");
		currentRoot.setComponentName(currentTable);
		currentRoot.setJavaTypeName(currentClassName);
		currentRoot.addProperty("extends", currentExtendedClassName);
		if (isToPersist(currentExtendedClassName)) {
			currentRoot.addProperty("persistentChild", "true");
		} else {
			currentRoot.addProperty("persistentChild", "false");
		}
		currentRoot.addProperty("major", "" + major);
		currentRoot.addProperty("minor", "" + minor);
		currentRoot.addProperty("database", database);

		Node currentAttributeNode = null;

		while (it.hasNext()) { //over alle attributen
			currentAttributeNode = (Node) it.next();
			//iterate over attributes			
			NamedNodeMap currentAttrMap = currentAttributeNode.getAttributes();
			//attributes of persistentAttribute-element

			attributeName = currentAttrMap.getNamedItem("name").getNodeValue();
			isAccessor = currentAttrMap.getNamedItem("accessor").getNodeValue();

			List currentAttributeContainedNodes =
				filterWhiteSpaceNodes(currentAttributeNode.getChildNodes());
			Iterator at = currentAttributeContainedNodes.iterator();
			Node getMethodNode = (Node) at.next();

			//getter			
			getMethodName =
				getMethodNode.getChildNodes().item(0).getNodeValue().trim();

			Node setMethodNode = (Node) at.next();

			//setter
			setMethodName =
				setMethodNode.getChildNodes().item(0).getNodeValue().trim();

			ContentTagList contentTagList = new ContentTagList();

			while (at.hasNext()) {
				contentTagList.addContentTag((Node) at.next());
			}

			getTables(this.currentRoot, contentTagList, true);

		}
		//		add "$"'s at beginning and end of all TableComponent-componentNames
		canonizeTableComponentNames(this.currentRoot);
		//		expliciet opgegeven table name aanvaarden indien korter dan een zekere limiet!
		if (tableNameProvided && (table.length() <= maxSizeTableName))
			currentRoot.addProperty("aliasName", table);
		//		add currentRoot to parsedPersistenceTrees (all attributes checked)			
		//addPersistenceTree(this.fullPrincipalClassName, this.currentRoot);

		return this.currentRoot;

	}

	/**
	 * Embeds all table names belonging to parent tree between "$"s and creates aliases.
	 * @param parent
	 */
	private void canonizeTableComponentNames(TableComponent parent) {
		String name = parent.getComponentName();
		if (!name.matches("\\$.*\\$")) {
			name = "$" + name + "$";
			parent.setComponentName(name);
		}
		parent.addProperty("aliasName", Utilities.createAlias(name));
		if (!parent.isLeaf()) {
			Collection coll = parent.getChildren();
			Iterator it = coll.iterator();
			while (it.hasNext()) {
				canonizeTableComponentNames((TableComponent) it.next());
			}
		}
	}

	/**
	 * Constructs table name by checking first for given alternative (table), otherwise making up a table name.
	 * @param currentClassName
	 * @param table without major/minor pversion
	 * @param major
	 * @param minor
	 * @return
	 */
	private String constructTableName(
		String currentClassName,
		String table,
		int major,
		int minor) {
		String currentTable;
		if (table.equals(""))
			currentTable =
				Utilities.constructTableName(currentClassName, major, minor);
		else
			currentTable = Utilities.constructTableName(table, major, minor);
		return currentTable;
	}

	/**
	 * Filters empty #text-nodes out of NodeList
	 * @param children
	 * @return
	 */
	private List filterWhiteSpaceNodes(NodeList children) {
		Node node;
		List nonWhiteSpaceNodes = new ArrayList();
		for (int i = 0;
			i < children.getLength();
			i++) { //look for node of currentClass
			node = children.item(i); //attribute/pversion/db-element
			String name = node.getNodeName();

			if ((!name.equals("#text")) && (!name.equals("#comment")))
				nonWhiteSpaceNodes.add(node);
		}
		return nonWhiteSpaceNodes;
	}

	/**
	 * Bouw TableComponent-tree op.
	 * @param parent
	 * @param contentTagList
	 */
	private void getTables(
		TableComponent parent,
		ContentTagList contentTagList,
		boolean childOfRoot)
		throws MissingContentTagException {
		Map contentTag = null;
		String compType = "";
		//dit is de compType uit xml-file, NIET compType uit propertyMap van een TableComponent!!
		String compTypeKey = "";
		String impClass = "";
		String impClassKey = "";
		int size = 0;
		int sizeKey = 0;
		int array = 0;
		String contained = "false";
		String containedKey = "false";
		try {
			contentTag = contentTagList.pullContentTag();
		} catch (IndexOutOfBoundsException e1) {
			throw new MissingContentTagException("No content tags left!");
		}

		compType = (String) contentTag.get("compType");
		compTypeKey = (String) contentTag.get("compTypeKey");
		impClass = (String) contentTag.get("impClass");
		impClassKey = (String) contentTag.get("impClassKey");
		//voorlopig niet gebruiken
		contained = (String) contentTag.get("contained");
		containedKey = (String) contentTag.get("containedKey");
		size = ((Integer) contentTag.get("size")).intValue();
		sizeKey = ((Integer) contentTag.get("sizeKey")).intValue();
		array = ((Integer) contentTag.get("array")).intValue();

		if (Utilities.isPrimitive(compType)) {
			//primitive (array) or String: contained heeft geen zin
			if (array > 0) { //primitive array
				TableComponent ar = new ArrayTable(getFKeys(parent), array);
				initChildOfRoot(childOfRoot, ar);
				ar.setComponentName(
					parent.getComponentName() + "_" + attributeName);
				ar.setJavaTypeName(compType); //type of content of array!!!
				parent.add(ar);
				if (addArrayString) {
					parent.addProperty(
						"compType",
						parent.getProperty("compType")
							+ Utilities.constructArrayHaakjes(array, ' '));
				}
				parent = ar;
				childOfRoot = false;
			}
			addArrayString = false;
			TableComponent attr = new Attribute();
			initChildOfRoot(childOfRoot, attr);
			attr.setComponentName(attributeName);
			attr.setJavaTypeName(compType);
			if (compType.equals("java.lang.String")
				|| compType.equals("String"))
				attr.addProperty("size", String.valueOf(size));
			parent.add(attr);

		} else if (Utilities.isCollection(compType)) { //Collection
			if (array > 0) { //array of Collections
				TableComponent ar = new ArrayTable(getFKeys(parent), array);
				initChildOfRoot(childOfRoot, ar);
				ar.setComponentName(
					parent.getComponentName() + "_" + attributeName);
				ar.setJavaTypeName(compType);
				//type of content of array!!!			
				parent.add(ar);
				if (addArrayString) {
					parent.addProperty(
						"compType",
						parent.getProperty("compType")
							+ Utilities.constructArrayHaakjes(array, ' '));
				}
				parent = ar;
				childOfRoot = false;
			}

			addArrayString = false;
			TableComponent coll = new CollectionTable(getFKeys(parent));
			initChildOfRoot(childOfRoot, coll);
			coll.setComponentName(
				parent.getComponentName() + "_" + attributeName);
			coll.setJavaTypeName(compType);
			if (!impClass.equals(""))
				coll.addProperty("impClass", impClass);
			try {
				Map tmp = contentTagList.peekContentTag();
				coll.addProperty("compType", (String) tmp.get("compType"));
			} catch (IndexOutOfBoundsException e) {
				throw new MissingContentTagException("Content tag te weinig!");
			}
			addArrayString = true;
			getTables(coll, contentTagList, false);
			parent.add(coll);
		} else if (Utilities.isMap(compType)) { //Map
			if (array > 0) { //array of Maps
				TableComponent ar = new ArrayTable(getFKeys(parent), array);
				initChildOfRoot(childOfRoot, ar);
				ar.setComponentName(
					parent.getComponentName() + "_" + attributeName);
				ar.setJavaTypeName(compType);
				//type of content of array!!!			
				parent.add(ar);
				if (addArrayString) {
					parent.addProperty(
						"compType",
						parent.getProperty("compType")
							+ Utilities.constructArrayHaakjes(array, ' '));
				}
				parent = ar;
				childOfRoot = false;
			}

			addArrayString = false;
			TableComponent map = new MapTable(getFKeys(parent));
			map.setComponentName(
				parent.getComponentName() + "_" + attributeName);
			map.setJavaTypeName(compType);
			initChildOfRoot(childOfRoot, map);
			if (!impClass.equals(""))
				map.addProperty("impClass", impClass);
			try {
				Map tmp = contentTagList.peekContentTag();
				map.addProperty("compType", (String) tmp.get("compType"));
				getTables(map, tmp); //key
			} catch (IndexOutOfBoundsException e) {
				throw new MissingContentTagException("Content tag te weinig!");
			}
			addArrayString = true;
			getTables(map, contentTagList, false);
			//value															
			parent.add(map);
		} else { //persistent object
			if (array > 0) { //array of persistent objects
				TableComponent ar = new ArrayTable(getFKeys(parent), array);
				initChildOfRoot(childOfRoot, ar);
				ar.setComponentName(
					parent.getComponentName() + "_" + attributeName);
				ar.setJavaTypeName(compType);
				//type of content of array!!!			
				parent.add(ar);
				if (addArrayString) {
					parent.addProperty(
						"compType",
						parent.getProperty("compType")
							+ Utilities.constructArrayHaakjes(array, ' '));
				}
				parent = ar;
				childOfRoot = false;
			}
			addArrayString = false;
			TableComponent ref = new ReferenceAttribute();
			initChildOfRoot(childOfRoot, ref);
			ref.setComponentName(attributeName);
			ref.setJavaTypeName("java.lang.String");
			ref.addProperty(
				"size",
				String.valueOf(TableComponent.getOIDLength()));
			ref.addProperty("referencedJavaTypeName", compType);
			ref.addProperty("contained", contained);
			if (impClass.equals("unknown implementor"))
				ref.addProperty("isInterface", "true");
			else if (impClass.equals("unknown extender"))
				ref.addProperty("isAbstract", "true");
			parent.add(ref);
		}

	}

	/**
	 * Get Collection of FK-(Reference)Attributes for children of parent.
	 * @param parent
	 * @return
	 */
	private Collection getFKeys(TableComponent parent) {
		ArrayList res = new ArrayList();
		Collection siblings = parent.getChildren();
		Iterator it = siblings.iterator();
		while (it.hasNext()) {
			TableComponent t = (TableComponent) it.next();
			String name = t.getComponentName();
			if (Boolean.valueOf(t.getProperty("key")).booleanValue()) {
				Class cla = null;
				try {
					cla = Class.forName("persistence.table.ReferenceAttribute");
				} catch (ClassNotFoundException e) {
					// nooit bereikt
					e.printStackTrace();
				}
				if (cla.isInstance(t)) { //reference attribute
					ReferenceAttribute ref = new ReferenceAttribute();
					ref.setComponentName("ref" + t.getComponentName());
					ref.setJavaTypeName(t.getJavaTypeName());
					ref.addProperty("size", t.getProperty("size"));
					ref.addProperty(
						"referencedJavaTypeName",
						t.getProperty("referencedJavaTypeName"));
					ref.addProperty("contained", t.getProperty("contained"));
					ref.addProperty("key", "true");
					ref.addProperty("FK", "true");
					res.add(ref);
				} else { //primitive
					String type = t.getJavaTypeName();
					Attribute attr = new Attribute();
					attr.setComponentName("ref" + t.getComponentName());
					attr.setJavaTypeName(type);
					if (type.equals("java.lang.String")
						|| type.equals("String"))
						attr.addProperty("size", t.getProperty("size"));
					attr.addProperty("key", "true");
					attr.addProperty("FK", "true");
					res.add(attr);
				}

			}
		}
		return res;
	}

	private void initChildOfRoot(boolean childOfRoot, TableComponent attr) {
		if (childOfRoot) {
			attr.addProperty("get", getMethodName);
			attr.addProperty("set", setMethodName);
			attr.addProperty("attributeName", attributeName);
			attr.addProperty("accessor", isAccessor);
		}
	}

	/**
	 * Add a KEY-field to a MapTable and add compType(Key)-properties to parent.
	 * @param parent
	 * @param contentTag
	 * @throws MissingContentTagException
	 */
	private void getTables(TableComponent parent, Map contentTag) {
		//String compType = (String) contentTag.get("compType");
		//dit is de compType uit xml-file, NIET compType uit propertyMap van een TableComponent!!
		String compTypeKey = (String) contentTag.get("compTypeKey");
		String impClassKey = (String) contentTag.get("impClassKey");
		String containedKey = (String) contentTag.get("containedKey");
		int sizeKey = ((Integer) contentTag.get("sizeKey")).intValue();

		parent.addProperty("compTypeKey", compTypeKey);
		parent.addProperty("impClassKey", impClassKey);

		if (Utilities.isPrimitive(compTypeKey)) {
			TableComponent attr = new Attribute();
			attr.setComponentName("KEY");
			attr.setJavaTypeName(compTypeKey);
			if (compTypeKey.equals("java.lang.String")
				|| compTypeKey.equals("String"))
				attr.addProperty("size", String.valueOf(sizeKey));
			attr.addProperty("key", "true");
			parent.add(attr);
		} else {
			if (!(Utilities.isCollection(compTypeKey)
				|| Utilities.isMap(
					compTypeKey))) { //compTypeKey is persistent object
				TableComponent ref = new ReferenceAttribute();
				ref.setComponentName("KEY");
				ref.addProperty("key", "true");
				ref.setJavaTypeName("java.lang.String");
				ref.addProperty(
					"size",
					String.valueOf(TableComponent.getOIDLength()));
				ref.addProperty("referencedJavaTypeName", compTypeKey);
				ref.addProperty("contained", containedKey);
				//OK: ref is zelf de key!
				ref.addProperty("key", "true");
				if (impClassKey.equals("unknown implementor"))
					ref.addProperty("isInterface", "true");
				else if (impClassKey.equals("unknown extender"))
					ref.addProperty("isAbstract", "true");
				parent.add(ref);

			}

		}

	}
	/**
	 * Collection contains all classes which will have to be persisted. This is 
	 * needed for inheritance checks.
	 * @param collection
	 */
	public static void setAllClassesToPersist(Collection collection) {
		allClassesToPersist = collection;
	}

	/**
	 * Checks whether fullClassName has to be persisted.
	 * @param fullClassName
	 * @return
	 */
	public static boolean isToPersist(String fullClassName) {
		return allClassesToPersist.contains(fullClassName);
	}

}
