/*
 * Created on 21-okt-2003
 *
*/
package persistence;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import persistence.type.ObjectID;

/**
 * Utility class.
 * @author bram
 *
 * 21-okt-2003
 */
public class Utilities {
	/**
	 * Checks whether type (can be with \"s) corresponds to java.lang.String, java.util.Date or a primitive type according to 
	 * {@link java.lang.Class#isPrimitive() java.lang.Class#isPrimitive()} or one of their Object counterparts.
	 * @param type
	 * @return
	 */
	public static boolean isPrimitive(String type) {
		type = removeAanhalingsTekens(type);
		return type.equals("int")
			|| type.equals("long")
			|| type.equals("float")
			|| type.equals("double")
			|| type.equals("boolean")
			|| type.equals("byte")
			|| type.equals("short")
			|| type.equals("char")
			|| type.equalsIgnoreCase("String")
			|| type.equalsIgnoreCase("java.lang.String")
			|| type.equalsIgnoreCase("Integer")
			|| type.equalsIgnoreCase("java.lang.Integer")
			|| type.equalsIgnoreCase("Float")
			|| type.equalsIgnoreCase("java.lang.Float")
			|| type.equalsIgnoreCase("Double")
			|| type.equalsIgnoreCase("java.lang.Double")
			|| type.equalsIgnoreCase("Boolean")
			|| type.equalsIgnoreCase("java.lang.Boolean")
			|| type.equalsIgnoreCase("Byte")
			|| type.equalsIgnoreCase("java.lang.Byte")
			|| type.equalsIgnoreCase("Short")
			|| type.equalsIgnoreCase("java.lang.Short")
			|| type.equalsIgnoreCase("Character")
			|| type.equalsIgnoreCase("java.lang.Character")
			|| type.equalsIgnoreCase("Date")
			|| type.equalsIgnoreCase("java.util.Date");

	}

	/**
	 * Verwijder aanhalingstekens aan beide kanten van String.
	 * @param s
	 * @return
	 */
	public static String removeAanhalingsTekens(String s) {
		if (s.matches("\".*\""))
			return s.substring(1, s.length() - 1);
		else
			return s;
	}

	/**
	 * Checks whether fullTypeName corresponds to array. Convention is that an array's name ends in "[]".
	 * @param fullTypeName
	 * @return
	 */
	public static boolean isArray(String fullTypeName) {
		return fullTypeName.endsWith("[]");
	}

	/**
	 * Check whether type (can be with \"s) is real primitive (Integer, String, Date, ... are NOT real primitive!).
	 * @param type
	 * @return
	 */
	public static boolean isRealPrimitive(String type) {
		type = Utilities.removeAanhalingsTekens(type);
		return type.equals("int")
			|| type.equals("long")
			|| type.equals("float")
			|| type.equals("double")
			|| type.equals("boolean")
			|| type.equals("byte")
			|| type.equals("short")
			|| type.equals("char");
	}

	/**
		 * Returns true if name is a Collection.
		 *
		 * @param name  The name of the type.
		 * @return      true/false				
		 */
	public static boolean isCollection(String typeName) {
		try {
			if ((!isArray(typeName) && (!isPrimitive(typeName)))) {
				typeName = Utilities.removeAanhalingsTekens(typeName);
				Class cla = Class.forName(typeName);
				Class colCla = Class.forName("java.util.Collection");
				return colCla.isAssignableFrom(cla);
			}
		} catch (ClassNotFoundException e) {
		} catch (NullPointerException e) {
		}
		return false;
	}

	/**
		 * Returns true if name is a Map.
		 *
		 * @param name  The name of the type.
		 * @return      true/false				
		 */
	public static boolean isMap(String typeName) {
		try {
			if ((!isArray(typeName) && (!isPrimitive(typeName)))) {
				typeName = Utilities.removeAanhalingsTekens(typeName);
				Class cla = Class.forName(typeName);
				Class mapCla = Class.forName("java.util.Map");
				return mapCla.isAssignableFrom(cla);
			}
		} catch (ClassNotFoundException e) {
		} catch (NullPointerException e) {
		}
		return false;
	}

	/**
	 * Gives dimension of array with arrayName. Convention is that arrays end in "[]", so basically the number of "[]"'s are counted.
	 * @param arrayName
	 * @return
	 */
	public static int getArrayDimension(String arrayName) {
		int arrayDimension = 0;
		String strippedArrayName = arrayName;
		while (Utilities.isArray(strippedArrayName)) {
			arrayDimension++;
			strippedArrayName =
				strippedArrayName.substring(0, strippedArrayName.length() - 2);
		}
		return arrayDimension;
	}

	/**
	 * Construct dimension of array out of a String like "[][]" (dimension 2).
	 * @param dim
	 * @return
	 */
	public static int getDimension(String dim) {
		StringBuffer buf = new StringBuffer(dim);
		int i = 0;
		while (buf.length() != 0) {
			buf.delete(0, 2);
			i++;
		}
		return i;
	}

	/**
	 * Construct Java bean getter. If type equals "boolean", "Boolean" or "java.lang.Boolean", one
	 * becomes is... in stead of get...
	 * @param name
	 * @param type
	 * @return
	 */
	public static String constructGetter(String name, String type) {
		if (type.equals("boolean")
			|| type.equals("Boolean")
			|| type.equals("java.lang.Boolean"))
			return "is"
				+ name.substring(0, 1).toUpperCase()
				+ name.substring(1, name.length());
		else
			return "get"
				+ name.substring(0, 1).toUpperCase()
				+ name.substring(1, name.length());
	}

	/**
	 * Construct Java bean setter.
	 * @param name
	 * @return
	 */
	public static String constructSetter(String name) {
		return "set"
			+ name.substring(0, 1).toUpperCase()
			+ name.substring(1, name.length());
	}

	/**
	 * Prints all system properties.
	 */
	public static void printSystemProperties() {
		Properties props = System.getProperties();
		Set set = props.entrySet();
		Iterator it = set.iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
			System.out.println(entry.getKey() + ":\t" + entry.getValue());
		}
	}

	public static String prepareAspectName(String type) {
		return type.substring(type.lastIndexOf('.') + 1) + "_aspect";
	}

	public static String constructArrayHaakjes(int dim, char c) {
		String res = "";
		for (int i = 0; i < dim; i++) {
			res += "[" + c + "]";
		}
		return res;
	}
	/**
	 * Vervang '?'s door tokens %index% met index startend vanaf startIndex
	 * @param s
	 * @param startIndex
	 * @return dimensie keer een token
	 */
	public static String tokenizeGradually(String s, int startIndex) {
		String res = s;
		int pos = res.indexOf('?');
		int index = startIndex;
		while (pos != -1) {
			res = res.replaceFirst("\\?", "%" + (index++) + "%");
			pos = res.indexOf('?');
		}
		return res;
	}

	/**
	 * Construct unqualified type name.
	 * @param type
	 * @return
	 */
	public static String getUnqualifiedTypeName(String type) {
		String[] split = type.split("\\.");
		return split[split.length - 1];
	}

	public static String getPackageName(String fullClassName) {
		int pos = fullClassName.lastIndexOf('.');
		if (pos == -1)
			return "";
		else
			return fullClassName.substring(0, pos);
	}

	/**
	 * Plaats "ref" voor alle keys.
	 * @param map
	 * @return
	 */
	public static Map referize(final Map map) {
		Map res = new HashMap();
		Iterator it = map.keySet().iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			res.put("$ref" + key.substring(1), (String) map.get(key));
		}
		return res;
	}

	/**
	 * Turn class name to inner class name, ...
	 * @param className
	 * @return
	 */
	public static String checkClassName(String className) {
		Class cla = null;
		boolean OK = false;
		do {
			try {
				cla = Class.forName(className);
				OK = true;
			} catch (ClassNotFoundException e) {
				className =
					className.substring(0, className.lastIndexOf('.'))
						+ "$"
						+ className.substring(
							className.lastIndexOf('.') + 1,
							className.length());
			}
		} while (!OK);

		return className;
	}

	/**
	 * Zet filler tussen Strings.
	 * @param strings
	 * @param filler
	 * @param endFiller
	 * @return
	 */
	public static String fillIn(
		Collection strings,
		String filler,
		boolean endFiller) {
		String res = "";
		Iterator it = strings.iterator();
		while (it.hasNext()) {
			res += (String) it.next() + filler;
		}
		if ((!endFiller) && (res.length() >= filler.length()))
			res = res.substring(0, res.length() - filler.length());
		return res;
	}

	public static String constructTableName(
		String className,
		int major,
		int minor) {
		return className.replaceAll("\\.", "_") + "$" + major + "$" + minor;
	}

	/**
	 * Get rid of "..._" and last "$".
	 * @param tableName
	 * @return
	 */
	public static String extractAttributeName(String tableName) {
		int index = tableName.lastIndexOf('_');
		if (index == -1)
			return tableName.substring(1, tableName.length() - 1);
		else
			return tableName.substring(index + 1, tableName.length() - 1);

	}

	/**
	 * Create short alias for long name.
	 * @param longName
	 * @return
	 */
	public static String createAlias(String longName) {
		return "alias" + Math.abs(longName.hashCode());
	}

	/**
	 * Determine whether cla is an inner class (non-static nested class).
	 * @param cla
	 * @return
	 */
	public static boolean isInner(Class cla) {
		boolean res = false;
		try {
			Class deCla = cla.getDeclaringClass();
			if ((deCla != null) && (!Modifier.isStatic(cla.getModifiers())))
				res = true;
		} catch (Exception e) {
		}
		return res;
	}

	/**
	 * Determine whether cla is a nested class.
	 * @param cla
	 * @return
	 */
	public static boolean isNested(Class cla) {
		return Modifier.isStatic(cla.getModifiers());
		//gewone static class kan niet!
	}

	/**
	 * Interprete value as a boolean value, otherwise return true.
	 * @param value
	 * @return
	 */
	public static boolean booleanValue(String value) {
		if (value.equalsIgnoreCase("true") || value.equalsIgnoreCase("false"))
			return Boolean.valueOf(value).booleanValue();
		else
			return true;
	}

	/**
	 * Create object with no-arg constructor. Type is extracted out of ObjectID and can be
	 * ordinary, inner or nested class.
	 * @param OID
	 * @return
	 */
	public static Object constructObject(ObjectID OID) {
		Object res = null;
		try {
			Class cla = Class.forName(OID.getAClass());
			Class deCla = cla.getDeclaringClass();
			if ((deCla != null)
				&& (!Modifier.isStatic(cla.getModifiers()))) { //inner class
				Constructor cons = cla.getConstructor(new Class[] { deCla });
				Object tmp = deCla.newInstance();
				res = cons.newInstance(new Object[] { tmp });
			} else {
				res = cla.newInstance();
			}
		} catch (InstantiationException e) {
			System.out.println("normaal niet mogelijk");
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			System.out.println("normaal niet mogelijk");
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			System.out.println("normaal niet mogelijk");
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
		return res;
	}
}
