/*
 * Created on 4-mrt-2004
 *
*/
package persistence.cache;

import persistence.cache.exceptions.AlreadyInCacheException;
import persistence.cache.exceptions.NotInCacheException;
import persistence.type.ObjectID;
import persistence.type.Persistent;

/**
 * Omschrijft gedrag van (simpele) cache.
 * @author bram
 *
 * 4-mrt-2004
 */
public interface Cache {
	/**
	 * Read deep object with OID.
	 * @param OID
	 * @return
	 * @throws CacheException
	 */
	public Object readDeep(ObjectID OID) throws NotInCacheException;
	
	/**
	 * Read shallow object with OID.
	 * @param OID
	 * @return
	 * @throws CacheException
	 */
	public Object readShallow(ObjectID OID) throws NotInCacheException;
	
	/**
	 * Write deep object with OID.
	 * @param ob
	 * @throws CacheException
	 */
	public void writeDeepToCache(ObjectID OID, Object ob) throws AlreadyInCacheException;
	
	/**
	 * Write shallow object with OID.
	 * @param ob
	 * @throws CacheException
	 */
	public void writeShallowToCache(ObjectID OID, Object ob) throws AlreadyInCacheException;
	
	/**
	 * Remove object with OID.
	 * @param OID
	 */
	public void remove(ObjectID OID);
	
	/**
	 * Is deep object met OID al gecached?
	 * @param OID
	 * @return
	 */
	public boolean checkForDeepOID(ObjectID OID);
	
	/**
	 * Is shallow object met OID al gecached?
	 * @param OID
	 * @return
	 */
	public boolean checkForShallowOID(ObjectID OID);
	
	/**
	 * Shows contents cache.
	 * @return
	 */
	public String toString();
}
