/*
 * Created on 4-mrt-2004
 *
*/
package persistence.cache;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import persistence.cache.exceptions.AlreadyInCacheException;
import persistence.cache.exceptions.CacheException;
import persistence.cache.exceptions.NotInCacheException;
import persistence.type.ObjectID;
import persistence.type.Persistent;

/**
 * Eenvoudige implementatie van cache. Idealiter ook reference count, ...
 * @author bram
 *
 * 4-mrt-2004
 */
public class SimpleCache implements Cache {
	private Map deepObjects; //OID->object
	private Map shallowObjects; //OID->object

	/**
	 * 
	 */
	public SimpleCache() {
		super();
		deepObjects = Collections.synchronizedMap(new HashMap());
		shallowObjects = Collections.synchronizedMap(new HashMap());
	}

	/**
	 * @see persistence.cache.Cache#readDeep(persistence.type.ObjectID)
	 */
	public synchronized Object readDeep(ObjectID OID)
		throws NotInCacheException {
		Object res = deepObjects.get(OID);
		if (res == null)
			throw new NotInCacheException("Requested deep object not in cache!");
		return res;
	}

	/**
	 * @see persistence.cache.Cache#readShallow(persistence.type.ObjectID)
	 */
	public synchronized Object readShallow(ObjectID OID)
		throws NotInCacheException {
		Object res = shallowObjects.get(OID);
		if (res == null)
			throw new NotInCacheException("Requested shallow object not in cache!");
		return res;
	}

	/**
	 * @see persistence.cache.Cache#writeDeepToCache(persistence.type.Persistent)
	 */
	public synchronized void writeDeepToCache(ObjectID OID, Object ob)
		throws AlreadyInCacheException {
		if (deepObjects.containsKey(OID))
			throw new AlreadyInCacheException("Deep object already in cache!");
		deepObjects.put(OID, ob);
	}

	/**
	 * @see persistence.cache.Cache#writeShallowToCache(persistence.type.Persistent)
	 */
	public synchronized void writeShallowToCache(ObjectID OID, Object ob)
		throws AlreadyInCacheException {
		if (shallowObjects.containsKey(OID))
			throw new AlreadyInCacheException("Shallow object already in cache!");
		shallowObjects.put(OID, ob);
	}

	/**
	 * @see persistence.cache.Cache#checkForDeepOID(persistence.type.ObjectID)
	 */
	public boolean checkForDeepOID(ObjectID OID) {
		return deepObjects.containsKey(OID);
	}

	/**
	 * @see persistence.cache.Cache#checkForShallowOID(persistence.type.ObjectID)
	 */
	public boolean checkForShallowOID(ObjectID OID) {
		return shallowObjects.containsKey(OID);
	}

	/**
	 * Cache bevat persistente objecten. Eens verwijderd uit de DB, blijft de data wel transint bestaan
	 * in de JVM, maar niet meer in de cache.
	 * @see persistence.cache.Cache#remove(persistence.type.ObjectID)
	 */
	public void remove(ObjectID OID) {
		System.out.println("removing " + OID + " ...");
		deepObjects.remove(OID);
		shallowObjects.remove(OID);
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "Shallow cache:\n"
			+ shallowObjects.toString()
			+ "\nDeep cache:\n"
			+ deepObjects.toString();
	}

}
