/*
 * Created on 21-okt-2003
 *
*/
package persistence.table;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import persistence.visitor.TableComponentVisitor;

/**
 * Composite die mapping van O/R-mapping voorstelt.
 * @author bram
 * @persistent
 * @pversion major="1"
 * 21-okt-2003
 */
public abstract class TableComponent {
	private String componentName; //name of component
	private String javaTypeName; //name of corresponding java type
	protected static final int OIDLength = 1000;

	/**
	 * Bevat eigenschappen van TableComponent:
	 * <ul>
	 * <li>root boolean (default false)-&gt;deprecated [Table]</li>
	 * <li>key boolean (default false) [TableComponent]</li>
	 * <li>FK boolean (default false) [TableComponent]</li>
	 * <li>contained String (default "false") [ReferenceAttribute]</li>
	 * <li>get String (no default; ZONDER "." en "()";enkel top level) [TableComponent]</li>
	 * <li>set String (no default; ZONDER "." en "()";enkel top level) [TableComponent]</li>
	 * <li>referencedComponentName String (no default)-&gt;deprecated [ReferenceAttribute]</li>
	 * <li>referencedJavaTypeName String (no default) [ReferenceAttribute]</li>
	 * <li>dimension int (no default) [ArrayTable]</li>
	 * <li>size int (default IN Table-hirarchie -1, in PDL 256) [TableComponent]</li>	 
	 * <li>attributeName String (no default) [Attribute]</li>
	 * <li>compType String (no default) [CollectionTable en MapTable]</li>
	 * <li>compTypeKey String (no default) [MapTable]</li>
	 * <li>impClass String (no default) [CollectionTable;MapTable]</li>
	 * <li>impClassKey String (no default)-&gt;deprecated  [MapTable]</li>
	 * <li>extends String (default "java.lang.Object") [Table]</li>
	 * <li>major int (NO default: verplicht) [Table]</li>
	 * <li>minor int (default 0) [Table]</li>
	 * <li>database String (no default) [Table]</li>
	 * <li>persistentChild boolean (true/false) [Table]</li>
	 * <li>accessor boolean (default false; OID true; refs false)</li>
	 * <li>index boolean (default false; OID true; refs false)</li>
	 * <li>aliasName (no default)</li>
	 * <li>isInterface boolean (default false)[RefAttribute]</li>
	 * <li>isAbstract boolean (default false)[RefAttribute]</li>
	 * </ul>
	 * 
	 * No default means null.
	 * @persistent
	 * @content compType="java.lang.String" compTypeKey="java.lang.String"
	 */
	private Map propertiesMap;

	/**
	 * Constructs unnamed, non-root and non-key TableComponent.	 
	 */
	protected TableComponent() {
		componentName = null;
		javaTypeName = null;
		propertiesMap = new HashMap();
	}

	/**
	 * 
	 * @param t
	 */
	public void add(TableComponent t) {
	}

	/**
	 * 
	 * @param t
	 */
	public void remove(TableComponent t) {
	}

	/**
	 * 
	 * @return
	 */
	public List getChildren() {
		return null;
	}

	/**
	 * 
	 * @return
	 */
	public String getComponentName() {
		return componentName;
	}

	/**
	 * @return
	 */
	public Map getPropertiesMap() {
		return propertiesMap;
	}

	/**
	 * @param map
	 */
	public void setPropertiesMap(Map map) {
		propertiesMap = map;
	}

	/**
	 * 
	 * @param key
	 * @param value
	 */
	public void addProperty(String key, String value) {
		propertiesMap.put(key, value);
	}

	/**
	 * Default value provided for "root"-key, "size"-key en "sizeKey"-key.
	 * @param key
	 * @return
	 */
	public String getProperty(String key) {
		String res = (String) propertiesMap.get(key);
		if (res == null) {
			if (key.equals("root")) {
				res = "false";
			} else {
				if (key.startsWith("size")) {
					res = "-1";
				} else {
					if (key.equals("accessor")) {
						res = "false";
					} else {
						if (key.equals("index")) {
							res = "false";
						}
					}
				}
			}
		}
		return res;
	}

	/**
	 * 
	 * @param key
	 * @return
	 */
	public boolean hasProperty(String key) {
		return propertiesMap.containsKey(key);
	}

	/**
	 * @param string
	 */
	public void setComponentName(String string) {
		componentName = string;
	}

	/**
	 * @return
	 */
	public boolean isLeaf() {
		return true;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String res = "";

		List chil = getChildren();
		Iterator it = chil.iterator();
		while (it.hasNext()) {
			res = res + ((TableComponent) it.next()).toString("");
		}

		return res;
	}

	private String toString(String dim) {
		String res = "";
		if (isLeaf())
			res = res + dim + getComponentName() + "\n";
		else {
			List chil = getChildren();
			Iterator it = chil.iterator();
			while (it.hasNext()) {
				res =
					res
						+ dim
						+ ((TableComponent) it.next()).toString(dim + "\t");
			}
		}
		return res;
	}

	public abstract void accept(TableComponentVisitor visitor);

	/**
	 * @return
	 */
	public String getJavaTypeName() {
		return javaTypeName;
	}

	/**
	 * @param string
	 */
	public void setJavaTypeName(String string) {
		javaTypeName = string;
	}

	/**
	 * @return
	 */
	public static int getOIDLength() {
		return OIDLength;
	}

}
