/*
 * Created on 11-apr-2004
 *
*/
package persistence.type;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

import persistence.DatabaseConnection;

/**
 * Factory for Connections.
 * @author bram
 *
 * 11-apr-2004
 */
public class ConnectionFactory {
	private static Properties props;

	/**
	 * File called db.properties contains all needed info for setting up Connections.
	 * @return
	 */
	public static Properties getProperties() {
		if (props == null) {
			props = new Properties();
			try {
				InputStream in =
					new BufferedInputStream(
						new FileInputStream("./db.properties"));
				props.load(in);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return props;
	}

	/**
	 * Obtain a Connection.
	 * @return null if something went wrong
	 */
	public Connection getConnection() {
		try {
			return new DatabaseConnection(getProperties()).getConn();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Signal to the factory that Connection isn't needed anymore.
	 * @param connection
	 */
	public void closeConnection(Connection connection) {
		System.out.println("@@@ConnectionFactory: Connection closing.");
		if (connection != null) {
			try {
				connection.close();
			} catch (SQLException ex) {
				System.err.println(
					"@@@ConnectionFactory: Connection did NOT close.");
				ex.printStackTrace();
			}
		}
	}

	/**
	 * Prohibit use of a factory outside persistence aspects.
	 */
	static aspect LimitUse {
		declare error : (
			execution(
				public void persistence.type.ConnectionFactory.closeConnection(
					java.sql.Connection))
				|| execution(
					public Connection persistence
						.type
						.ConnectionFactory
						.getConnection()))
			&& !(within(aspects..*)
				|| within(
					persistence.type.
						* )) : "Only open/close with ConnectionFactory in the persistence aspects!";
	}

}
