/*
 * Created on 26-apr-2003
*/
package persistence.type;

import java.io.Serializable;
import java.util.Date;

/**
 * Single object which creates the {@link persistence.ObjectID ObjectID's}.
 * @author bram
 *
 *26-apr-2003
*/
public class ObjectIDFactory implements Serializable {
	/**
		 * Implementation of {@link persistence.ObjectID ObjectID}.
		 * @author bram		 
		 *4-mei-2003
		 */
	public class ObjectIDImpl implements ObjectID {
		private Date time; //timestamp
		private String ip; //ip-adres
		private String aClass; //actual class (fixed)
		//aanduiding wat klasse is van object met dit ObjectIDImpl

		/**
		 * Creates instance of {@link persistence.ObjectID ObjectID}.
		 */
		public ObjectIDImpl(String aClass) {
			time = new Date();
			try {
				ip = java.net.InetAddress.getLocalHost().getHostAddress();
			} catch (java.net.UnknownHostException uhe) {
				ip = "0.0.0.0";
			}

			this.aClass = aClass;
		}

		/**
		 * Creates instance of {@link persistence.ObjectID ObjectID}.
		 */
		public ObjectIDImpl(Date time, String ip, String aClass) {
			this.time = time;
			this.ip = ip;
			this.aClass = aClass;
		}

		/**
		 * 
		 * @see persistence.type.ObjectID#getAClass()
		 */
		public String getAClass() {
			return aClass;
		}

		/**
		 * @return timestamp			 
		*/
		public Date getTime() {
			return time;
		}

		/**
		 * @return IP address			 
		*/
		public String getIp() {
			return ip;
		}

		/**
		 * Time, IP and actual class (NO current class).
		 */
		public String toString() {
			return ""
				+ time.getTime()
				+ "___"
				+ ip.replaceAll("\\.", "_")
				+ "___"
				+ aClass.replaceAll("\\.", "_");
		}

		/** 
		 * Current class distinguishes also!
		 * @see java.lang.Comparable#compareTo(java.lang.Object)			 
		 */
		public int compareTo(Object o) {
			String temp = ((ObjectID) o).toString();
			return toString().compareTo(temp);
		}

		/** 
		 * @see java.lang.Object#equals(java.lang.Object)		 
		 */
		public boolean equals(Object o) {
			System.out.println("EQUALS van OID");
			return this.compareTo(o) == 0;
		}

		/**
		 * @see java.lang.Object#hashCode()
		 */
		public int hashCode() {
			return toString().hashCode();
		}

	}

	/**
	 * 
	 *
	 */
	public ObjectIDFactory() {
	}

	/**
	 * Produce a {@link persistence.ObjectID ObjectID} for Object.
	 * @return {@link persistence.ObjectID ObjectID}	 
	*/
	public ObjectID getObjectID(Object o) {
		//to make keys different for same class, same IP and "chain creation"
		for (int i = 0; i < 1000; i++) {
			Math.sin(Math.cos((new Double(i + "." + i)).doubleValue()));
		}
		return new ObjectIDImpl(o.getClass().getName());
	}

	/**
		 * Produce a {@link persistence.ObjectID ObjectID} for Object of class fullClassName.
		 * @return {@link persistence.ObjectID ObjectID}	 
		*/
	public ObjectID getObjectID(String fullClassName) {
		return new ObjectIDImpl(fullClassName);
	}

	/**
	 * Produce a {@link persistence.ObjectID ObjectID}
	 * @param s
	 * @return {@link persistence.ObjectID ObjectID}	 
	*/
	public ObjectID translateToObjectID(String s) {
		String[] rij = s.split("___");
		return new ObjectIDImpl(
			new Date(Long.parseLong(rij[0])),
			rij[1].replaceAll("_", "."),
			rij[2].replaceAll("_", "."));
	}

}
