/*
 * Created on 3-nov-2003
 *
*/
package persistence.visitor;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import persistence.table.*;

import net.sf.hibernate.HibernateException;
import net.sf.hibernate.sql.Create;
import net.sf.hibernate.sql.Insert;

/**
 * Genereren van code voor creren nodige tabellen uit databank.
 * @author bram
 *
 * 3-nov-2003
 */
public class CreateVisitor implements TableComponentVisitor {
	private ArrayList stack; /*contains ArrayLists with Creates*/
	private String connectionExpression;
	private String dialectVariableName;
	private Create currentCreate;
	private String resultStrings;
	private final String JDBCStatementName = "stmt";
	private final String JDBCPreparedStatementName = "prepStmt";

	/**
	 * 
	 */
	public CreateVisitor(
		String connectionExpression,
		String dialectVariableName) {
		this.connectionExpression = connectionExpression;
		this.dialectVariableName = dialectVariableName;
		currentCreate = null;
		stack = new ArrayList();
		resultStrings = "";
	}

	/**
	 * @see persistence.TableComponentVisitor#visitAttribute(persistence.Attribute)
	 */
	public void visitAttribute(Attribute attr) {
		if (Boolean.valueOf(attr.getProperty("key")).booleanValue()) {
			currentCreate.addPrimaryColumn(
				attr.getComponentName(),
				attr.getJavaTypeName(),
				Integer.parseInt(attr.getProperty("size")));
		} else {
			currentCreate.addColumn(
				attr.getComponentName(),
				attr.getJavaTypeName(),
				Integer.parseInt(attr.getProperty("size")));
		}
	}

	/**
	 * @see persistence.TableComponentVisitor#visitReferenceAttribute(persistence.ReferenceAttribute)
	 */
	public void visitReferenceAttribute(ReferenceAttribute refAttr) {
		if (Boolean.valueOf(refAttr.getProperty("key")).booleanValue()) {
			//"$KEY$
			currentCreate.addPrimaryColumn(
				refAttr.getComponentName(),
				refAttr.getJavaTypeName(),
				Integer.parseInt(refAttr.getProperty("size")));
		} else {
			currentCreate.addColumn(
				refAttr.getComponentName(),
				refAttr.getJavaTypeName(),
				Integer.parseInt(refAttr.getProperty("size")));
		}
	}

	/**
	 * @see persistence.TableComponentVisitor#visitTable(persistence.Table)
	 */
	public void visitTable(Table table) {
		Insert nullInsert = new Insert(null); //altijd null-object inserten
		nullInsert.setTableName(table.getProperty("aliasName"));
		currentCreate = new Create(this.dialectVariableName);
		currentCreate.setTableName(table.getProperty("aliasName"));
		List children = table.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			TableComponent tab = (TableComponent) it.next();
			tab.accept(this);
			if (tab.getComponentName().equals("$OID$")
				|| persistence.table.ReferenceAttribute.class.isInstance(tab))
				nullInsert.addColumn(tab.getComponentName(), "'null'");
		}
		try {
			resultStrings =
				resultStrings
					+ JDBCStatementName
					+ "="
					+ connectionExpression
					+ ".createStatement();\n"
					+ JDBCStatementName
					+ ".executeUpdate(\""
					+ currentCreate.getCreateStatement()
					+ "\");\n"
					+ JDBCStatementName
					+ ".executeUpdate(\""
					+ currentCreate.getPKStatement()
					+ "\");\n"
					+ JDBCStatementName
					+ ".executeUpdate(\""
					+ nullInsert.toStatementString()
					+ "\");\n"
					+ JDBCStatementName
					+ ".close();\n";
		} catch (HibernateException e) {
			//never reached???
			e.printStackTrace();
		}

	}

	/**
	 * @see persistence.TableComponentVisitor#visitArrayTable(persistence.ArrayTable)
	 */
	public void visitArrayTable(ArrayTable ar) {
		stack.add(currentCreate);
		currentCreate = new Create(this.dialectVariableName);
		currentCreate.setTableName(ar.getProperty("aliasName"));
		List children = ar.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}
		try {
			resultStrings =
				resultStrings
					+ JDBCStatementName
					+ "="
					+ connectionExpression
					+ ".createStatement();\n"
					+ JDBCStatementName
					+ ".executeUpdate(\""
					+ currentCreate.getCreateStatement()
					+ "\");\n"
					+ JDBCStatementName
					+ ".executeUpdate(\""
					+ currentCreate.getPKStatement()
					+ "\");\n"
					+ JDBCStatementName
					+ ".close();\n";
		} catch (HibernateException e) {
			//never reached???
			e.printStackTrace();
		}
		currentCreate = (Create) stack.remove(stack.size() - 1);
	}

	/**
	 * @see persistence.TableComponentVisitor#visitCollectionTable(persistence.CollectionTable)
	 */
	public void visitCollectionTable(CollectionTable coll) {
		stack.add(currentCreate);
		currentCreate = new Create(this.dialectVariableName);
		currentCreate.setTableName(coll.getProperty("aliasName"));
		List children = coll.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}
		try {
			resultStrings =
				resultStrings
					+ JDBCStatementName
					+ "="
					+ connectionExpression
					+ ".createStatement();\n"
					+ JDBCStatementName
					+ ".executeUpdate(\""
					+ currentCreate.getCreateStatement()
					+ "\");\n"
					+ JDBCStatementName
					+ ".executeUpdate(\""
					+ currentCreate.getPKStatement()
					+ "\");\n"
					+ JDBCStatementName
					+ ".close();\n";
		} catch (HibernateException e) {
			//never reached???
			e.printStackTrace();
		}
		currentCreate = (Create) stack.remove(stack.size() - 1);

	}

	/**
	 * @see persistence.TableComponentVisitor#visitMapTable(persistence.MapTable)
	 */
	public void visitMapTable(MapTable map) {
		stack.add(currentCreate);
		currentCreate = new Create(this.dialectVariableName);
		currentCreate.setTableName(map.getProperty("aliasName"));
		List children = map.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}
		try {
			resultStrings =
				resultStrings
					+ JDBCStatementName
					+ "="
					+ connectionExpression
					+ ".createStatement();\n"
					+ JDBCStatementName
					+ ".executeUpdate(\""
					+ currentCreate.getCreateStatement()
					+ "\");\n"
					+ JDBCStatementName
					+ ".executeUpdate(\""
					+ currentCreate.getPKStatement()
					+ "\");\n"
					+ JDBCStatementName
					+ ".close();\n";
		} catch (HibernateException e) {
			//never reached???
			e.printStackTrace();
		}
		currentCreate = (Create) stack.remove(stack.size() - 1);

	}

	/**
	 * @see persistence.TableComponentVisitor#getResult()
	 */
	public String getResult() {
		return resultStrings;
	}

}
