/*
 * Created on 1-nov-2003
 *
*/
package persistence.visitor;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import persistence.Utilities;
import persistence.table.*;

import net.sf.hibernate.sql.Delete;

/**
 * Creren van advice-code voor verwijderen van @persistent-objecten uit databank.
 * @author bram
 *
 * 1-nov-2003
 */
public class DeleteVisitor implements TableComponentVisitor {
	private int nameDiscriminator = 0;
	//used for creating aliases: in stead of obj.getName().getFirstName(), one uses obj35.getFirstName()
	private int tokenIndex = 0;
	private String fullClassName;
	private ArrayList stack;
	/*contains ArrayLists with elements:
		0: unfinished Delete-object
		1: invocations needed to obtain Object corresponding to (part of) attribute described by element 0
		2: alias for current Object
		3: ArrayList with names of primary keys
		4: ArrayList with values of primary keys
		5: keyAlias for Map (if applicable)
		6: iterationNeeded
		*/
	private ArrayList resultStrings;
	//extra indentation, blocks, ... can be added in getresult() + declaration van stmt, ...(<-->transaction, ... in AbstractSQLFactoryImpl
	private Delete currentDelete;
	private String currentAttribute;
	private String currentOID;
	private String currentAlias;
	private String nextAlias;
	private String keyAlias;
	private ArrayList keyNames;
	private ArrayList keyValues;
	private ArrayList deleteStrings;
	private boolean iterationNeeded;
	//moeten componenten van child array/Collection/Map apart verwijderd worden?
	private String subTreeSQL; //SQL-code for subtree of main table
	private final String JDBCStatementName = "stmt";
	private final String JDBCPreparedStatementName = "prepStmt";
	private final String iteratorName = "it";
	private final String nameOfPersistentObject = "obj";
	private String connectionExpression;

	/**
	 * 
	 */
	public DeleteVisitor(String fullClassName, String connectionExpression) {
		this.fullClassName = fullClassName;
		this.connectionExpression = connectionExpression;
		stack = new ArrayList();
		currentDelete = null; //initialized in visitTable
		currentAttribute = nameOfPersistentObject;
		currentOID = this.nameOfPersistentObject + ".getOID().toString()";
		//initialized in visitAttribute
		currentAlias = ""; //initialized in visitTable
		nextAlias = ""; //initialized in visitTable
		keyAlias = ""; //zie Map
		keyNames = null; //initialized in visitTable
		keyValues = null; //initialized in visitTable
		resultStrings = new ArrayList();
		iterationNeeded = false;
		deleteStrings = new ArrayList();
	}

	/**
	 * @see persistence.TableComponentVisitor#visitAttribute(persistence.Attribute)
	 */
	public void visitAttribute(Attribute attr) {
		if (attr.getComponentName().matches("\\$.*OID\\$")) {
			keyNames.add(attr.getComponentName());
			keyValues.add("'\"+" + currentOID + "+\"'");
			//let op de ' voor/na \"!!!			

		}
		return;
	}

	/**
	 * @see persistence.TableComponentVisitor#visitReferenceAttribute(persistence.ReferenceAttribute)
	 */
	public void visitReferenceAttribute(ReferenceAttribute refAttr) {
		if (Utilities.booleanValue(refAttr.getProperty("contained"))) {
			//contained=>no own identity=>under control
			subTreeSQL =
				subTreeSQL
					+ "if("
					+ nextAlias
					+ "!=null) "
					+ nextAlias
					+ ".delete();\n";
			iterationNeeded = true;
		} else {
			subTreeSQL =
				subTreeSQL
					+ "//"
					+ refAttr.getComponentName()
					+ " is NOT contained, so it's NOT deleted!\n";
		}

	}

	/**
	 * @see persistence.TableComponentVisitor#visitTable(persistence.Table)
	 */
	public void visitTable(Table table) {
		currentAlias = currentAttribute; //"obj"+(nameDiscriminator++);
		currentDelete = new Delete();
		currentDelete.setTableName(table.getProperty("aliasName"));
		keyNames = new ArrayList();
		keyValues = new ArrayList();
		List children = table.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			subTreeSQL = new String("");
			/*
			 * subTreeSQL will be passed to deeper level as long as there are Maps/Collections/Arrays.
			 */
			TableComponent tab = (TableComponent) it.next();
			nextAlias = currentAlias + "." + tab.getProperty("get") + "()";
			tab.accept(this);
			if (iterationNeeded)
				resultStrings.add(subTreeSQL);
			iterationNeeded = false;
		}
		currentDelete.setPrimaryKeyColumnNames(
			(String[]) keyNames.toArray(new String[keyNames.size()]),
			(String[]) keyValues.toArray(new String[keyValues.size()]));

		deleteStrings.add(currentDelete.toStatementString());
	}

	/**
	 * @see persistence.TableComponentVisitor#visitArrayTable(persistence.ArrayTable)
	 */
	public void visitArrayTable(ArrayTable ar) {
		tokenIndex++;
		int index = tokenIndex;
		//stack current state
		ArrayList stackEntry = new ArrayList();
		stackEntry.add(currentDelete);
		stackEntry.add(currentAttribute);
		stackEntry.add(currentAlias);
		stackEntry.add(keyNames);
		stackEntry.add(keyValues);
		stackEntry.add(new Boolean(iterationNeeded));
		stack.add(stackEntry);

		//create new state
		iterationNeeded = false;
		keyNames = new ArrayList();
		keyValues = new ArrayList();
		currentAlias = "obj" + (nameDiscriminator++);
		currentAttribute = nextAlias;

		currentDelete = new Delete();
		currentDelete.setTableName(ar.getProperty("aliasName"));
		List children = ar.getChildren();
		Iterator it = children.iterator();
		String haakjes =
			Utilities.tokenizeGradually(
				Utilities.constructArrayHaakjes(
					(new Integer(ar.getProperty("dimension"))).intValue(),
					'?'),
				index);
		nextAlias = currentAlias + haakjes;
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}
		currentDelete.setPrimaryKeyColumnNames(
			(String[]) keyNames.toArray(new String[keyNames.size()]),
			(String[]) keyValues.toArray(new String[keyValues.size()]));

		//ANDERE volgorde dan in Delete en Insert, omdat huidige Delete geldt voor alle indices HUIDIGE array<-->subTreeSQL heeft andere waarden voor alle indices HUIDIGE array
		//1: sub aanpassen met while-lus					  
		if (iterationNeeded) { //enkel dan subTreeSQL aanpassen	
			String rijExpr = currentAlias;
			String forLus = "";
			String endForLus = "";
			for (int i = index;
				i
					< index
						+ (new Integer(ar.getProperty("dimension"))).intValue();
				i++) {
				forLus += "for(int i"
					+ i
					+ "=0;i"
					+ i
					+ "<"
					+ rijExpr
					+ ".length;i"
					+ i
					+ "++){\n";
				endForLus += "}\n";
				rijExpr += "[i" + i + "]";
				subTreeSQL = subTreeSQL.replaceAll("%" + i + "%", "i" + i);
			}
			subTreeSQL =
				ar.getJavaTypeName()
					+ Utilities.constructArrayHaakjes(
						(new Integer(ar.getProperty("dimension"))).intValue(),
						' ')
					+ " "
					+ currentAlias
					+ "="
					+ currentAttribute
					+ ";\n"
					+ forLus
					+ subTreeSQL
					+ endForLus;
		}

		//2: Insert achteraan subTree				
		deleteStrings.add(currentDelete.toStatementString());

		//unstack previous state
		stackEntry = (ArrayList) stack.remove(stack.size() - 1);
		currentDelete = (Delete) stackEntry.get(0);
		currentAttribute = (String) stackEntry.get(1);
		currentAlias = (String) stackEntry.get(2);
		keyNames = (ArrayList) stackEntry.get(3);
		keyValues = (ArrayList) stackEntry.get(4);
		iterationNeeded =
			iterationNeeded || ((Boolean) stackEntry.get(5)).booleanValue();

	}

	/**
	 * @see persistence.TableComponentVisitor#visitCollectionTable(persistence.CollectionTable)
	 */
	public void visitCollectionTable(CollectionTable coll) {
		tokenIndex++;
		int index = tokenIndex;
		//stack current state
		ArrayList stackEntry = new ArrayList();
		stackEntry.add(currentDelete);
		stackEntry.add(currentAttribute);
		stackEntry.add(currentAlias);
		stackEntry.add(keyNames);
		stackEntry.add(keyValues);
		stackEntry.add(new Boolean(iterationNeeded));
		stack.add(stackEntry);

		//create new state
		iterationNeeded = false;
		keyNames = new ArrayList();
		keyValues = new ArrayList();
		currentAlias = "obj" + (nameDiscriminator++);
		currentAttribute = nextAlias;

		currentDelete = new Delete();
		currentDelete.setTableName(coll.getProperty("aliasName"));
		List children = coll.getChildren();
		Iterator it = children.iterator();
		String currentIterator = iteratorName + index;
		nextAlias =
			"(("
				+ coll.getProperty("compType")
				+ ")"
				+ currentIterator
				+ ".next())";
		//currentAlias+".get(%"+index+"%)";
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}
		currentDelete.setPrimaryKeyColumnNames(
			(String[]) keyNames.toArray(new String[keyNames.size()]),
			(String[]) keyValues.toArray(new String[keyValues.size()]));
		//		ANDERE volgorde dan in Delete en Insert, omdat huidige Delete geldt voor alle indices HUIDIGE array<-->subTreeSQL heeft andere waarden voor alle indices HUIDIGE array
		//		1: sub aanpassen met while-lus					  
		if (iterationNeeded) { //enkel hier subTreeSQL aanpassen
			subTreeSQL =
				coll.getJavaTypeName()
					+ " "
					+ currentAlias
					+ "="
					+ currentAttribute
					+ ";\n"
					+ "Iterator "
					+ currentIterator
					+ "="
					+ currentAlias
					+ ".iterator();\n"
					+ "int i"
					+ index
					+ "=0;\n"
					+ "while("
					+ currentIterator
					+ ".hasNext()){\n"
					+ subTreeSQL.replaceAll("%" + index + "%", "i" + index)
					+ "i"
					+ index
					+ "++;\n"
					+ "}\n";
		}
		//2: Insert achteraan subTree				
		deleteStrings.add(currentDelete.toStatementString());

		//unstack previous state
		stackEntry = (ArrayList) stack.remove(stack.size() - 1);
		currentDelete = (Delete) stackEntry.get(0);
		currentAttribute = (String) stackEntry.get(1);
		currentAlias = (String) stackEntry.get(2);
		keyNames = (ArrayList) stackEntry.get(3);
		keyValues = (ArrayList) stackEntry.get(4);
		iterationNeeded =
			iterationNeeded || ((Boolean) stackEntry.get(5)).booleanValue();

	}

	/**
	 * @see persistence.TableComponentVisitor#visitMapTable(persistence.MapTable)
	 */
	public void visitMapTable(MapTable map) {
		tokenIndex++;
		int index = tokenIndex;
		//stack current state
		ArrayList stackEntry = new ArrayList();
		stackEntry.add(currentDelete);
		stackEntry.add(currentAttribute);
		stackEntry.add(currentAlias);
		stackEntry.add(keyNames);
		stackEntry.add(keyValues);
		stackEntry.add(keyAlias);
		stackEntry.add(new Boolean(iterationNeeded));
		stack.add(stackEntry);

		//create new state
		iterationNeeded = false;
		keyNames = new ArrayList();
		keyValues = new ArrayList();
		String oldAlias = currentAlias;
		currentAlias = "obj" + (nameDiscriminator++); //value		
		keyAlias = "obj" + (nameDiscriminator++); //key
		currentAttribute = nextAlias;

		currentDelete = new Delete();
		currentDelete.setTableName(map.getProperty("aliasName"));
		List children = map.getChildren();
		Iterator it = children.iterator();
		String currentIterator = iteratorName + index;
		nextAlias =
			"(("
				+ map.getProperty("compType")
				+ ")"
				+ currentAlias
				+ ".get("
				+ keyAlias
				+ "))";
		//currentAlias+".get(%"+index+"%)";
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}
		currentDelete.setPrimaryKeyColumnNames(
			(String[]) keyNames.toArray(new String[keyNames.size()]),
			(String[]) keyValues.toArray(new String[keyValues.size()]));
		//		ANDERE volgorde dan in Delete en Insert, omdat huidige Delete geldt voor alle indices HUIDIGE array<-->subTreeSQL heeft andere waarden voor alle indices HUIDIGE array

		//		1: sub aanpassen met while-lus					  
		if (iterationNeeded) { //enkel hier subTreeSQL aanpassen
			subTreeSQL =
				map.getJavaTypeName()
					+ " "
					+ currentAlias
					+ "="
					+ currentAttribute
					+ ";\n"
					+ "Iterator "
					+ currentIterator
					+ "="
					+ currentAlias
					+ ".keySet().iterator();\n"
					+ "while("
					+ currentIterator
					+ ".hasNext()){\n"
					+ map.getProperty("compTypeKey")
					+ " "
					+ keyAlias
					+ "=(("
					+ map.getProperty("compTypeKey")
					+ ")"
					+ currentIterator
					+ ".next());\n"
					+ subTreeSQL.replaceAll("%" + index + "%", nextAlias)
					+ 
				//subTreeSQL+
	"}\n";
		}
		//2: Insert achteraan subTree				
		deleteStrings.add(currentDelete.toStatementString());

		//unstack previous state
		stackEntry = (ArrayList) stack.remove(stack.size() - 1);
		currentDelete = (Delete) stackEntry.get(0);
		currentAttribute = (String) stackEntry.get(1);
		currentAlias = (String) stackEntry.get(2);
		keyNames = (ArrayList) stackEntry.get(3);
		keyValues = (ArrayList) stackEntry.get(4);
		keyAlias = (String) stackEntry.get(5);
		iterationNeeded =
			iterationNeeded || ((Boolean) stackEntry.get(6)).booleanValue();
	}

	/**
	 * @see persistence.TableComponentVisitor#getResult()
	 */
	public String getResult() {
		//declare stmt and prepStmt (stmt must be closed each time to allow new resultSet to be opened)
		String res =
			"pointcut deleter_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "("
				+ fullClassName
				+ " "
				+ nameOfPersistentObject
				+ "): this("
				+ nameOfPersistentObject
				+ ") && execution(public void persistence.type.Persistent.delete() throws persistence.exceptions.DeleteException);\n"
				+ "after("
				+ fullClassName
				+ " "
				+ nameOfPersistentObject
				+ ") throws persistence.exceptions.DeleteException: deleter_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "("
				+ nameOfPersistentObject
				+ "){\n"
				+ "ConnectionFactory conFac=new ConnectionFactory();\n"
				+ "Connection "
				+ connectionExpression
				+ "=conFac.getConnection();\n"
				+ "try{\n";
		res += getAllLines(JDBCStatementName);
		res += JDBCStatementName
			+ ".close();\n"
			+ "}catch(Exception e){\n"
			+ "throw new persistence.exceptions.DeleteException("
			+ nameOfPersistentObject
			+ ".getOID().toString(),e);\n"
			+ "}finally{\n"
			+ "conFac.closeConnection("
			+ connectionExpression
			+ ");\n"
			+ "}\n";
		return res + "}\n";
	}

	String getAllLines(String jdbcStatementName) {
		String res = "";
		for (int i = 0; i < resultStrings.size(); i++) {
			res = res + resultStrings.get(i);
		}
		Iterator it = deleteStrings.iterator();
		if (it.hasNext())
			res += "Statement "
				+ jdbcStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n";
		while (it.hasNext()) {
			res += jdbcStatementName
				+ ".executeUpdate(\""
				+ (String) it.next()
				+ "\");\n";
		}
		return res;
	}

}
