/*
 * Created on 3-nov-2003
 *
*/
package persistence.visitor;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import persistence.table.*;

import net.sf.hibernate.sql.Drop;

/**
 * Genereren van code voor verwijderen nodige tabellen uit databank.
 * @author bram
 *
 * 3-nov-2003
 */
public class DropVisitor implements TableComponentVisitor {
	private ArrayList stack; /*contains ArrayLists with Creates*/
	private String connectionExpression;
	private String dialectVariableName;
	private Drop currentDrop;
	private String resultStrings;
	private final String JDBCStatementName = "stmt";
	private final String JDBCPreparedStatementName = "prepStmt";

	/**
	 * 
	 */
	public DropVisitor(
		String connectionExpression,
		String dialectVariableName) {
		this.connectionExpression = connectionExpression;
		this.dialectVariableName = dialectVariableName;
		currentDrop = null;
		stack = new ArrayList();
		resultStrings = "";
	}

	/**
	 * @see persistence.TableComponentVisitor#visitAttribute(persistence.Attribute)
	 */
	public void visitAttribute(Attribute attr) {
		// NOTHING!

	}

	/**
	 * @see persistence.TableComponentVisitor#visitReferenceAttribute(persistence.ReferenceAttribute)
	 */
	public void visitReferenceAttribute(ReferenceAttribute refAttr) {
		// no rights to drop external table, because there might be other, not-contained objects stored in it

	}

	/**
	 * @see persistence.TableComponentVisitor#visitTable(persistence.Table)
	 */
	public void visitTable(Table table) {
		currentDrop = new Drop(this.dialectVariableName);
		currentDrop.setTableName(table.getProperty("aliasName"));
		List children = table.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}

		resultStrings =
			resultStrings
				+ JDBCStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n"
				+ JDBCStatementName
				+ ".executeUpdate(\""
				+ currentDrop.getDropStatement()
				+ "\");\n"
				+ JDBCStatementName
				+ ".close();\n";
	}

	/**
	 * @see persistence.TableComponentVisitor#visitArrayTable(persistence.ArrayTable)
	 */
	public void visitArrayTable(ArrayTable ar) {
		stack.add(currentDrop);
		currentDrop = new Drop(this.dialectVariableName);
		currentDrop.setTableName(ar.getProperty("aliasName"));
		List children = ar.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}

		resultStrings =
			resultStrings
				+ JDBCStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n"
				+ JDBCStatementName
				+ ".executeUpdate(\""
				+ currentDrop.getDropStatement()
				+ "\");\n"
				+ JDBCStatementName
				+ ".close();\n";

		currentDrop = (Drop) stack.remove(stack.size() - 1);

	}

	/**
	 * @see persistence.TableComponentVisitor#visitCollectionTable(persistence.CollectionTable)
	 */
	public void visitCollectionTable(CollectionTable coll) {
		stack.add(currentDrop);
		currentDrop = new Drop(this.dialectVariableName);
		currentDrop.setTableName(coll.getProperty("aliasName"));
		List children = coll.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}

		resultStrings =
			resultStrings
				+ JDBCStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n"
				+ JDBCStatementName
				+ ".executeUpdate(\""
				+ currentDrop.getDropStatement()
				+ "\");\n"
				+ JDBCStatementName
				+ ".close();\n";
		currentDrop = (Drop) stack.remove(stack.size() - 1);
	}

	/**
	 * @see persistence.TableComponentVisitor#visitMapTable(persistence.MapTable)
	 */
	public void visitMapTable(MapTable map) {
		stack.add(currentDrop);
		currentDrop = new Drop(this.dialectVariableName);
		currentDrop.setTableName(map.getProperty("aliasName"));
		List children = map.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			((TableComponent) it.next()).accept(this);
		}

		resultStrings =
			resultStrings
				+ JDBCStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n"
				+ JDBCStatementName
				+ ".executeUpdate(\""
				+ currentDrop.getDropStatement()
				+ "\");\n"
				+ JDBCStatementName
				+ ".close();\n";

		currentDrop = (Drop) stack.remove(stack.size() - 1);

	}

	/**
	 * @see persistence.TableComponentVisitor#getResult()
	 */
	public String getResult() {
		return resultStrings;
	}

}
