/*
 * Created on 30-okt-2003
 *
*/
package persistence.visitor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import persistence.Utilities;
import persistence.table.*;

import net.sf.hibernate.sql.Insert;

/**
 * {@link TableComponentVisitor TableComponentVisitor} which creates necessary SQL-code to 
 * insert object whose relational DB-representation is the visited TableComponent-hierarchy.
 * @author bram
 *
 * 30-okt-2003
 */
public class InsertVisitor implements TableComponentVisitor {
	private String fullClassName;
	private int nameDiscriminator = 0;
	//used for creating aliases: in stead of obj.getName().getFirstName(), one uses obj35.getFirstName()
	private int tokenIndex = 0;
	private ArrayList stack;
	/*contains ArrayLists with elements:
		0: unfinished Insert-object
		1: invocations needed to obtain Object corresponding to (part of) attribute described by element 0
		2: alias for current Object
		3: alias for key of Map
		4: keyMap
		*/
	private ArrayList resultStrings;
	//extra indentation, blocks, ... can be added in getresult() + declaration van stmt, ...(<-->transaction, ... in AbstractSQLFactoryImpl
	private Insert currentInsert;
	private String currentAttribute;
	private String currentAlias;
	private String keyAlias;
	private String nextAlias;
	private Map keyMap;
	private String subTreeSQL; //SQL-code for subtree of main table
	private final String JDBCStatementName = "stmt";
	private final String JDBCPreparedStatementName = "prepStmt";
	private final String iteratorName = "it";
	private String connectionExpression;
	private final String nameOfPersistentObject = "obj";

	public InsertVisitor(String fullClassName, String connectionExpression) {
		this.fullClassName = fullClassName;
		this.connectionExpression = connectionExpression;
		stack = new ArrayList();
		currentInsert = null; //initialized in visitTable
		currentAttribute = nameOfPersistentObject;
		currentAlias = ""; //initialized in visitTable
		nextAlias = ""; //initialized in visitTable
		resultStrings = new ArrayList();
		keyMap = new HashMap();
	}

	/**
	 * @see persistence.TableComponentVisitor#visitAttribute(persistence.Attribute)
	 */
	public void visitAttribute(Attribute attr) {
		//currentAlias=nextAlias;anders globale currentAlias kwijt
		if (attr.getComponentName().equals("$OID$")) {
			keyMap.put(
				attr.getComponentName(),
				"'\"+" + nextAlias + ".toString()" + "+\"'");
			currentInsert.addColumn(
				attr.getComponentName(),
				"'\"+" + nextAlias + ".toString()" + "+\"'");
			//let op de ' voor/na \"!!!
			return;
		} else if (
			attr.getComponentName().matches(
				"\\$ref.*\\$")) { //alle keys beginnend met ref
			//al aanwezig in Map
			currentInsert.addColumn(
				attr.getComponentName(),
				((String) keyMap.get(attr.getComponentName())));
			return;
		} else if (attr.getComponentName().matches("\\$POSITION\\d*\\$")) {
			//werk hier met index++ vr gebruik!!!
			int index = tokenIndex++;
			keyMap.put(attr.getComponentName(), "\"+%" + index + "%+\"");
			currentInsert.addColumn(
				attr.getComponentName(),
				"\"+%" + index + "%+\"");
			return;
		} else if (
			attr.getComponentName().equals("$KEY$")) { //no persistent object
			//werk hier met index++ vr gebruik!!!
			int index = tokenIndex++;
			tokenIndex++; //nodig omdat 2 objecten (key en value)
			keyMap.put(attr.getComponentName(), "'\"+" + keyAlias + "+\"'");
			if (attr.getJavaTypeName().equals("String")
				|| attr.getJavaTypeName().equals("java.lang.String")) {
				currentInsert.addColumn(
					attr.getComponentName(),
					"'\"+" + keyAlias + "+\"'");
				//let op de ' voor/na \"!!!
			} else {
				currentInsert.addColumn(
					attr.getComponentName(),
					"\"+" + keyAlias + "+\"");
			}
			return;
		}
		if (attr.getJavaTypeName().equals("String")
			|| attr.getJavaTypeName().equals("java.lang.String")) {
			currentInsert.addColumn(
				attr.getComponentName(),
				"'\"+" + nextAlias + "+\"'");
			//let op de ' voor/na \"!!!
		} else {
			currentInsert.addColumn(
				attr.getComponentName(),
				"\"+" + nextAlias + "+\"");
		}
	}

	/**
	 * @see persistence.TableComponentVisitor#visitReferenceAttribute(persistence.ReferenceAttribute)
	 */
	public void visitReferenceAttribute(ReferenceAttribute refAttr) {
		//currentAlias=nextAlias;anders globale currentAlias kwijt
		if (Utilities.booleanValue(refAttr.getProperty("contained")))
			subTreeSQL =
				subTreeSQL
					+ "if("
					+ nextAlias
					+ "!=null) "
					+ nextAlias
					+ ".write();\n";
		if (refAttr.getComponentName().equals("$KEY$")) {
			//persistent object as key of map
			keyMap.put(
				refAttr.getComponentName(),
				"'\"+(("
					+ keyAlias
					+ "!=null)?"
					+ keyAlias
					+ ".getOID().toString():\"null\")+\"'");
			currentInsert.addColumn(
				refAttr.getComponentName(),
				"'\"+(("
					+ keyAlias
					+ "!=null)?"
					+ keyAlias
					+ ".getOID().toString():\"null\")+\"'");
			//let op de ' voor/na \"!!!
		} else {
			if (refAttr.getComponentName().matches("\\$ref.*\\$")) {
				currentInsert.addColumn(
					refAttr.getComponentName(),
					((String) keyMap.get(refAttr.getComponentName())));
				//let op de ' voor/na \"!!!
			} else {
				currentInsert.addColumn(
					refAttr.getComponentName(),
					"'\"+(("
						+ nextAlias
						+ "!=null)?"
						+ nextAlias
						+ ".getOID().toString():\"null\")+\"'");
				//let op de ' voor/na \"!!!
			}
		}
	}

	/**
	 * Root table!!!
	 * @see persistence.TableComponentVisitor#visitTable(persistence.Table)
	 */
	public void visitTable(Table table) {
		currentAlias = currentAttribute; //"obj"+(nameDiscriminator++);		
		currentInsert = new Insert(null);
		currentInsert.setTableName(table.getProperty("aliasName"));
		List children = table.getChildren();
		Iterator it = children.iterator();
		while (it.hasNext()) {
			subTreeSQL = new String("");
			/*
			 * subTreeSQL will be passed to deeper level as long as there are Maps/Collections/Arrays. 
			 */
			TableComponent tab = (TableComponent) it.next();
			nextAlias = currentAlias + "." + tab.getProperty("get") + "()";
			tab.accept(this);
			resultStrings.add(subTreeSQL);
		}
		String res =
			JDBCStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n"
				+ JDBCStatementName
				+ ".executeUpdate(\""
				+ currentInsert.toStatementString()
				+ "\");\n"
				+ JDBCStatementName
				+ ".close();\n";
		resultStrings.add(res);
	}

	/**
	 * @see persistence.TableComponentVisitor#visitArrayTable(persistence.ArrayTable)
	 */
	public void visitArrayTable(ArrayTable ar) {
		int index = tokenIndex;
		//stack current state
		ArrayList stackEntry = new ArrayList(4);
		stackEntry.add(currentInsert);
		stackEntry.add(currentAttribute);
		stackEntry.add(currentAlias);
		stackEntry.add(keyMap);
		stack.add(stackEntry);

		//create new state
		keyMap = Utilities.referize(keyMap);
		currentAlias = "obj" + (nameDiscriminator++); //nieuwe naam		
		currentAttribute = nextAlias;
		//uitdrukking die gegeven wordt aan currentAlias
		currentInsert = new Insert(null);
		currentInsert.setTableName(ar.getProperty("aliasName"));
		List children = ar.getChildren();
		Iterator it = children.iterator();
		String haakjes =
			Utilities.tokenizeGradually(
				Utilities.constructArrayHaakjes(
					(new Integer(ar.getProperty("dimension"))).intValue(),
					'?'),
				index);
		nextAlias = currentAlias + haakjes;
		while (it.hasNext()) {
			TableComponent tab = (TableComponent) it.next();
			tab.accept(this);
		}

		//1: Insert achteraan subTree				
		subTreeSQL =
			subTreeSQL
				+ JDBCStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n"
				+ JDBCStatementName
				+ ".executeUpdate(\""
				+ currentInsert.toStatementString()
				+ "\");\n"
				+ "stmt.close();\n";
		//2: sub aanpassen met while-lus
		String rijExpr = currentAlias;
		String forLus = "";
		String endForLus = "";
		for (int i = index;
			i < index + (new Integer(ar.getProperty("dimension"))).intValue();
			i++) {
			forLus += "for(int i"
				+ i
				+ "=0;i"
				+ i
				+ "<"
				+ rijExpr
				+ ".length;i"
				+ i
				+ "++){\n";
			endForLus += "}\n";
			rijExpr += "[i" + i + "]";
			subTreeSQL = subTreeSQL.replaceAll("%" + i + "%", "i" + i);
		}
		subTreeSQL = forLus + subTreeSQL + endForLus;

		//3: declaratie van alias vooraan subTree (na herstellen token)
		subTreeSQL =
			ar.getJavaTypeName()
				+ Utilities.constructArrayHaakjes(
					(new Integer(ar.getProperty("dimension"))).intValue(),
					' ')
				+ " "
				+ currentAlias
				+ "="
				+ currentAttribute
				+ ";\n"
				+ "if("
				+ currentAlias
				+ "!=null){\n"
				+ subTreeSQL
				+ "}\n";

		//unstack previous state
		stackEntry = (ArrayList) stack.remove(stack.size() - 1);
		currentInsert = (Insert) stackEntry.get(0);
		currentAttribute = (String) stackEntry.get(1);
		currentAlias = (String) stackEntry.get(2);
		keyMap = (Map) stackEntry.get(3);
	}

	/**
	 * @see persistence.TableComponentVisitor#getResult()
	 */
	public String getResult() {
		String res =
			"pointcut writer_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "("
				+ fullClassName
				+ " "
				+ nameOfPersistentObject
				+ "): this("
				+ nameOfPersistentObject
				+ ") && execution(public void persistence.type.Persistent.write() throws persistence.exceptions.WriteException);\n"
				+ "after("
				+ fullClassName
				+ " "
				+ nameOfPersistentObject
				+ ")  throws persistence.exceptions.WriteException: writer_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "("
				+ nameOfPersistentObject
				+ "){\n"
				+ "ConnectionFactory conFac=new ConnectionFactory();\n"
				+ "Connection "
				+ connectionExpression
				+ "=conFac.getConnection();\n"
				+ "try{\n"
				+ "Statement "
				+ JDBCStatementName
				+ "=null;\n"
				+ "Iterator "
				+ iteratorName
				+ "=null;\n";
		for (int i = 0; i < resultStrings.size(); i++) {
			res = res + resultStrings.get(i);
		}
		res += "}catch(Exception e){\n"
			+ "throw new persistence.exceptions.WriteException("
			+ nameOfPersistentObject
			+ ".getOID().toString(),e);\n"
			+ "}finally{\n"
			+ "conFac.closeConnection("
			+ connectionExpression
			+ ");\n"
			+ "}\n";
		return res + "}\n";
	}

	public void visitCollectionTable(CollectionTable coll) {
		int index = tokenIndex;
		//stack current state
		ArrayList stackEntry = new ArrayList(4);
		stackEntry.add(currentInsert);
		stackEntry.add(currentAttribute);
		stackEntry.add(currentAlias);
		stackEntry.add(keyMap);
		stack.add(stackEntry);

		//create new state
		keyMap = Utilities.referize(keyMap);
		currentAlias = "obj" + (nameDiscriminator++);
		currentAttribute = nextAlias;

		currentInsert = new Insert(null);
		currentInsert.setTableName(coll.getProperty("aliasName"));
		List children = coll.getChildren();
		Iterator it = children.iterator();
		String currentIterator = iteratorName + index;
		String aliasForNextAlias = "al" + (nameDiscriminator++);
		nextAlias = aliasForNextAlias;
		while (it.hasNext()) {
			TableComponent tab = (TableComponent) it.next();
			tab.accept(this);
		}
		//1: Insert achteraan subTree				
		subTreeSQL =
			subTreeSQL
				+ JDBCStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n"
				+ JDBCStatementName
				+ ".executeUpdate(\""
				+ currentInsert.toStatementString()
				+ "\");\n"
				+ "stmt.close();\n";
		//		2: sub aanpassen met while-lus					  
		subTreeSQL =
			"Iterator "
				+ currentIterator
				+ "="
				+ currentAlias
				+ ".iterator();\n"
				+ "int i"
				+ index
				+ "=0;\n"
				+ "while("
				+ currentIterator
				+ ".hasNext()){\n"
				+ coll.getProperty("compType")
				+ " "
				+ aliasForNextAlias
				+ "=(("
				+ coll.getProperty("compType")
				+ ")"
				+ currentIterator
				+ ".next());\n"
				+ subTreeSQL.replaceAll("%" + index + "%", "i" + index)
				+ 
			//"(("+coll.getProperty("compType")+")"+currentIterator+".next())")+
	"i" + index + "++;\n" + "}\n";
		//		3: declaratie van alias vooraan subTree (na herstellen token)
		subTreeSQL =
			coll.getJavaTypeName()
				+ " "
				+ currentAlias
				+ "="
				+ currentAttribute
				+ ";\n"
				+ "if("
				+ currentAlias
				+ "!=null){\n"
				+ subTreeSQL
				+ "}\n";

		//unstack previous state
		stackEntry = (ArrayList) stack.remove(stack.size() - 1);
		currentInsert = (Insert) stackEntry.get(0);
		currentAttribute = (String) stackEntry.get(1);
		currentAlias = (String) stackEntry.get(2);
		keyMap = (Map) stackEntry.get(3);
	}

	public void visitMapTable(MapTable map) {
		int index = tokenIndex;
		//stack current state
		ArrayList stackEntry = new ArrayList(5);
		stackEntry.add(currentInsert);
		stackEntry.add(currentAttribute);
		stackEntry.add(currentAlias);
		stackEntry.add(keyAlias);
		stackEntry.add(keyMap);
		stack.add(stackEntry);

		//create new state
		keyMap = Utilities.referize(keyMap);
		currentAlias = "obj" + (nameDiscriminator++); //value
		keyAlias = "obj" + (nameDiscriminator++); //key
		currentAttribute = nextAlias;
		currentInsert = new Insert(null);
		currentInsert.setTableName(map.getProperty("aliasName"));
		List children = map.getChildren();
		Iterator it = children.iterator();
		String currentIterator = iteratorName + index;
		nextAlias =
			"(("
				+ map.getProperty("compType")
				+ ")"
				+ currentAlias
				+ ".get("
				+ keyAlias
				+ "))";
		//currentAlias+".get(%"+index+"%)";		
		while (it.hasNext()) {
			TableComponent tab = (TableComponent) it.next();
			tab.accept(this);
		}
		//1: Insert achteraan subTree				
		subTreeSQL =
			subTreeSQL
				+ JDBCStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n"
				+ JDBCStatementName
				+ ".executeUpdate(\""
				+ currentInsert.toStatementString()
				+ "\");\n"
				+ "stmt.close();\n";
		//		2: sub aanpassen met while-lus					  		
		subTreeSQL =
			"Iterator "
				+ currentIterator
				+ "="
				+ currentAlias
				+ ".keySet().iterator();\n"
				+ "while("
				+ currentIterator
				+ ".hasNext()){\n"
				+ map.getProperty("compTypeKey")
				+ " "
				+ keyAlias
				+ "=(("
				+ map.getProperty("compTypeKey")
				+ ")"
				+ currentIterator
				+ ".next());\n"
				+ subTreeSQL.replaceAll("%" + index + "%", nextAlias)
				+ "}\n";
		//		3: declaratie van alias vooraan subTree (na herstellen token)
		subTreeSQL =
			map.getJavaTypeName()
				+ " "
				+ currentAlias
				+ "="
				+ currentAttribute
				+ ";\n"
				+ "if("
				+ currentAlias
				+ "!=null){\n"
				+ subTreeSQL
				+ "}\n";

		//unstack previous state
		stackEntry = (ArrayList) stack.remove(stack.size() - 1);
		currentInsert = (Insert) stackEntry.get(0);
		currentAttribute = (String) stackEntry.get(1);
		currentAlias = (String) stackEntry.get(2);
		keyAlias = (String) stackEntry.get(3);
		keyMap = (Map) stackEntry.get(4);
	}
}
