/*
 * Created on 13-mrt-2004
 *
*/
package persistence.visitor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import persistence.Utilities;
import persistence.table.ArrayTable;
import persistence.table.Attribute;
import persistence.table.CollectionTable;
import persistence.table.MapTable;
import persistence.table.ReferenceAttribute;
import persistence.table.Table;
import persistence.table.TableComponent;

/**
 * Codegeneratie van type-advice voor queries.
 * @author bram
 *
 * 13-mrt-2004
 */
public class QueryCompTypeVisitor implements TableComponentVisitor {
	private String fullClassName;
	private final String mem = "mem";
	private final String attr = "attr";
	private final String thisType = "thisType";
	private final String realAttrName = "realAttrName";
	private ArrayList codePieces;
	private String codePiece;

	public QueryCompTypeVisitor(String fullClassName) {
		this.fullClassName = fullClassName;
		this.codePieces = new ArrayList();
		this.codePiece = "";
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#getResult()
	 */
	public String getResult() {
		String res =
			"pointcut getCompTypes_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "(\n"
				+ "MyMemoryQuery "
				+ mem
				+ ",\n"
				+ "String "
				+ attr
				+ ",\n"
				+ "String "
				+ thisType
				+ ") : this(\n"
				+ mem
				+ ")\n"
				+ "&& call(\n"
				+ "private void com.sun.jdori.common.query.MyMemoryQuery.getCompTypes(\n"
				+ "String,\n"
				+ "String))\n"
				+ "&& args("
				+ attr
				+ ", "
				+ thisType
				+ ")\n"
				+ "&& if ("
				+ thisType
				+ ".equals(\""
				+ fullClassName
				+ "\"));\n"
				+ "after(MyMemoryQuery "
				+ mem
				+ ", String "
				+ attr
				+ ", String "
				+ thisType
				+ ") : getCompTypes_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "(\n"
				+ mem
				+ ",\n"
				+ attr
				+ ",\n"
				+ thisType
				+ ") {\n"
				+ "String "
				+ realAttrName
				+ " = \"$\"+"
				+ attr
				+ ".substring("
				+ attr
				+ ".lastIndexOf(\".\") + 1)+\"$\";\n"
				+ "//zeker niet leeg\n";

		Iterator it = codePieces.iterator();
		while (it.hasNext()) {
			res += (String) it.next();
		}

		return res + "}\n";
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitArrayTable(persistence.table.ArrayTable)
	 */
	public void visitArrayTable(ArrayTable ar) {
		codePiece =
			"if ("
				+ realAttrName
				+ ".equals(\"$"
				+ Utilities.extractAttributeName(ar.getComponentName())
				+ "$\")) {\n"
				+ "Vector res = new Vector();\n"
				+ "res.add(\""
				+ ar.getProperty("compType")
				+ "\");\n"
				+ mem
				+ ".nameToCompTypeMapping.put("
				+ attr
				+ ", res);\n"
				+ "}else ";
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitAttribute(persistence.table.Attribute)
	 */
	public void visitAttribute(Attribute attri) {
		codePiece =
			"if ("
				+ realAttrName
				+ ".equals(\""
				+ attri.getComponentName()
				+ "\")) {\n"
				+ "Vector res = new Vector();\n"
				+ "res.add(\""
				+ attri.getJavaTypeName()
				+ "\");\n"
				+ mem
				+ ".nameToCompTypeMapping.put("
				+ attr
				+ ", res);\n"
				+ "}else ";

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitCollectionTable(persistence.table.CollectionTable)
	 */
	public void visitCollectionTable(CollectionTable coll) {
		codePiece =
			"if ("
				+ realAttrName
				+ ".equals(\"$"
				+ Utilities.extractAttributeName(coll.getComponentName())
				+ "$\")) {\n"
				+ "Vector res = new Vector();\n"
				+ "res.add(\""
				+ coll.getProperty("compType")
				+ "\");\n"
				+ mem
				+ ".nameToCompTypeMapping.put("
				+ attr
				+ ", res);\n"
				+ "}else ";
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitMapTable(persistence.table.MapTable)
	 */
	public void visitMapTable(MapTable map) {
		// TODO: later

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitReferenceAttribute(persistence.table.ReferenceAttribute)
	 */
	public void visitReferenceAttribute(ReferenceAttribute refAttr) {
		codePiece =
			"if ("
				+ realAttrName
				+ ".equals(\""
				+ refAttr.getComponentName()
				+ "\")) {\n"
				+ "Vector res = new Vector();\n"
				+ "res.add(\""
				+ refAttr.getProperty("referencedJavaTypeName")
				+ "\");\n"
				+ mem
				+ ".nameToCompTypeMapping.put("
				+ attr
				+ ", res);\n"
				+ "}else ";

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitTable(persistence.table.Table)
	 */
	public void visitTable(Table table) {
		Collection coll = table.getChildren();
		Iterator it = coll.iterator();
		while (it.hasNext()) {
			codePiece = ""; //doorgegeven naar elk child
			TableComponent tab = (TableComponent) it.next();
			if (!(Boolean.valueOf(tab.getProperty("key"))).booleanValue())
				tab.accept(this);
			codePieces.add(codePiece);
		}

		codePieces.add(
			"{ //this/variable\n"
				+ "Vector res = new Vector();\n"
				+ "res.add(\""
				+ fullClassName
				+ "\");\n"
				+ mem
				+ ".nameToCompTypeMapping.put("
				+ attr
				+ ", res);\n}\n");

	}

}
