/*
 * Created on 13-mrt-2004
 *
*/
package persistence.visitor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import persistence.Utilities;
import persistence.table.ArrayTable;
import persistence.table.Attribute;
import persistence.table.CollectionTable;
import persistence.table.MapTable;
import persistence.table.ReferenceAttribute;
import persistence.table.Table;
import persistence.table.TableComponent;

/**
 * Codegeneratie van join-advice voor queries.
 * @author bram
 *
 * 13-mrt-2004
 */
public class QueryJoinVisitor implements TableComponentVisitor {
	private String fullClassName;
	private final String mem = "mem";
	private final String attr = "attr";
	private final String thisType = "thisType";
	private final String foreignAttr = "foreignAttr";
	private final String realAttrName = "realAttrName";
	private final String thisAlias = "thisAlias";
	private final String outer = "outer";
	private ArrayList codePieces;
	private String codePiece;

	public QueryJoinVisitor(String fullClassName) {
		this.fullClassName = fullClassName;
		this.codePieces = new ArrayList();
		this.codePiece = "";
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#getResult()
	 */
	public String getResult() {
		String res =
			"pointcut getJoins_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "(\n"
				+ "MyMemoryQuery "
				+ mem
				+ ",\n"
				+ "String "
				+ attr
				+ ",\n"
				+ "String "
				+ thisType
				+ ",\n"
				+ "String "
				+ foreignAttr
				+ ",\n"
				+ "boolean "
				+ outer
				+ ") : this(\n"
				+ mem
				+ ")\n"
				+ "&& call(\n"
				+ "	private void com.sun.jdori.common.query.MyMemoryQuery.getJoins(\n"
				+ "String,\n"
				+ "String,\n"
				+ "String,\n"
				+ "boolean))\n"
				+ "&& args("
				+ attr
				+ ", "
				+ thisType
				+ ", "
				+ foreignAttr
				+ ", "
				+ outer
				+ ")\n"
				+ "&& if ("
				+ thisType
				+ ".equals(\""
				+ fullClassName
				+ "\"));\n\n"
				+ "	/**\n"
				+ "* Enkel Collections, arrays, Maps en persistent <--> anders negeren\n"
				+ "*/\n"
				+ "after(\n"
				+ "MyMemoryQuery "
				+ mem
				+ ",\n"
				+ "String "
				+ attr
				+ ",\n"
				+ "String "
				+ thisType
				+ ",\n"
				+ "String "
				+ foreignAttr
				+ ",\n"
				+ "boolean "
				+ outer
				+ ") : getJoins_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "(\n"
				+ mem
				+ ",\n"
				+ attr
				+ ",\n"
				+ thisType
				+ ",\n"
				+ foreignAttr
				+ ",\n"
				+ outer
				+ ") {\n"
				+ "//nesting: ontrafel echte naam attribuut door aliassen achterwaarts op te zoeken\n"
				+ "String "
				+ realAttrName
				+ " = \"$\"+"
				+ attr
				+ ".substring("
				+ attr
				+ ".lastIndexOf(\".\") + 1)+\"$\";\n"
				+ "String "
				+ thisAlias
				+ "="
				+ attr
				+ ".substring(0,"
				+ attr
				+ ".lastIndexOf(\".\"));\n"
				+ "//zeker niet leeg\n";

		Iterator it = codePieces.iterator();
		while (it.hasNext()) {
			res += (String) it.next();
		}
		res = res.substring(0, res.length() - 5); //remove trailing "else "

		res += "\n}\n";
		return res;
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitArrayTable(persistence.table.ArrayTable)
	 */
	public void visitArrayTable(ArrayTable ar) {
		codePiece += "if("
			+ realAttrName
			+ ".equals(\""
			+ "$"
			+ Utilities.extractAttributeName(ar.getComponentName())
			+ "$"
			+ "\")) { //collection\n"
			+ "if ("
			+ foreignAttr
			+ " == null)\n"
			+ "if (!"
			+ outer
			+ ")\n"
			+ mem
			+ ".joinSet.add(\n"
			+ "(String) "
			+ mem
			+ ".nameToAliasMapping.get("
			+ attr
			+ ")\n"
			+ "+ \".$refOID$=\"\n"
			+ "+ "
			+ thisAlias
			+ "\n"
			+ "+ \".$OID$\");\n"
			+ "else {\n"
			+ "String attrAlias =\n"
			+ "(String) "
			+ mem
			+ ".nameToAliasMapping.get("
			+ attr
			+ ");\n"
			+ mem
			+ ".outerJoinAliases.add(attrAlias);\n"
			+ mem
			+ ".outerJoinAliases.add("
			+ thisAlias
			+ ");\n"
			+ "Vector tmp = new Vector();\n"
			+ "tmp.add("
			+ thisAlias
			+ ");\n"
			+ "tmp.add(\n"
			+ "attrAlias\n"
			+ "+ \".$OID$=\"\n"
			+ "+ "
			+ thisAlias
			+ "\n"
			+ "+ \".$OID$\");\n"
			+ mem
			+ ".outerJoins.put(attrAlias, tmp);\n"
			+ "}\n"
			+ "//projects is naam attribuut alle FK's; verondersteld dat compType Persistent is (geen nesting en primitives al uitgefilterd)\n"
			+ "else\n"
			+ mem
			+ ".joinSet.add(\n"
			+ "(String) "
			+ mem
			+ ".nameToAliasMapping.get("
			+ attr
			+ ")\n"
			+ "+\".$"
			+ Utilities.extractAttributeName(ar.getComponentName())
			+ "$\"\n"
			+ "+ \"=\"\n"
			+ "+ (String) "
			+ mem
			+ ".nameToAliasMapping.get(\n"
			+ foreignAttr
			+ ")\n"
			+ "+ \".$OID$\");\n"
			+ "//contains, ...\n"
			+ "}else ";

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitAttribute(persistence.table.Attribute)
	 */
	public void visitAttribute(Attribute attr) {
		//NIETS

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitCollectionTable(persistence.table.CollectionTable)
	 */
	public void visitCollectionTable(CollectionTable coll) {
		codePiece += "if("
			+ realAttrName
			+ ".equals(\"$"
			+ Utilities.extractAttributeName(coll.getComponentName())
			+ "$\")) { //collection\n"
			+ "if ("
			+ foreignAttr
			+ " == null)\n"
			+ 
		//gewone joins
		"if (!"
			+ outer
			+ ")\n"
			+ mem
			+ ".joinSet.add(\n"
			+ "(String) "
			+ mem
			+ ".nameToAliasMapping.get("
			+ attr
			+ ")\n"
			+ "+ \".$refOID$=\"\n"
			+ "+ "
			+ thisAlias
			+ "\n"
			+ "+ \".$OID$\");\n"
			+ "else {\n"
			+ "String attrAlias =\n"
			+ "(String) "
			+ mem
			+ ".nameToAliasMapping.get("
			+ attr
			+ ");\n"
			+ mem
			+ ".outerJoinAliases.add(attrAlias);\n"
			+ mem
			+ ".outerJoinAliases.add("
			+ thisAlias
			+ ");\n"
			+ "Vector tmp = new Vector();\n"
			+ "tmp.add("
			+ thisAlias
			+ ");\n"
			+ "tmp.add(\n"
			+ "attrAlias\n"
			+ "+ \".$OID$=\"\n"
			+ "+ "
			+ thisAlias
			+ "\n"
			+ "+ \".$OID$\");\n"
			+ mem
			+ ".outerJoins.put(attrAlias, tmp);\n"
			+ "}\n"
			+ "else\n"
			+ mem
			+ ".joinSet.add(\n"
			+ "(String) "
			+ mem
			+ ".nameToAliasMapping.get("
			+ attr
			+ ")\n"
			+ "+\".$"
			+ Utilities.extractAttributeName(coll.getComponentName())
			+ "$\"\n"
			+ "+ \"=\"\n"
			+ "+ (String) "
			+ mem
			+ ".nameToAliasMapping.get(\n"
			+ foreignAttr
			+ ")\n"
			+ "+ \".$OID$\");\n"
			+ "//contains, ...\n"
			+ "}else ";

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitMapTable(persistence.table.MapTable)
	 */
	public void visitMapTable(MapTable map) {
		// TODO: voorlopig niet

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitReferenceAttribute(persistence.table.ReferenceAttribute)
	 */
	public void visitReferenceAttribute(ReferenceAttribute refAttr) {
		codePiece += "if ("
			+ realAttrName
			+ ".equals(\""
			+ refAttr.getComponentName()
			+ "\")) { //persistent\n"
			+ "//realAttrName moet vervangen worden door naam attribuut; enkel $OID$; foreignAttr altijd null (Persistent kan niets containen)\n"
			+ mem
			+ ".joinSet.add(\n"
			+ thisAlias
			+ "+\".\"+"
			+ realAttrName
			+ "\n"
			+ "+ \"=\"\n"
			+ "+ (String) "
			+ mem
			+ ".nameToAliasMapping.get("
			+ attr
			+ ")\n"
			+ "+ \".$OID$\");\n"
			+ "//linking attribute with own class table\n"
			+ "}else ";
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitTable(persistence.table.Table)
	 */
	public void visitTable(Table table) {
		Collection coll = table.getChildren();
		Iterator it = coll.iterator();
		while (it.hasNext()) {
			codePiece = ""; //doorgegeven naar elk child
			TableComponent tab = (TableComponent) it.next();
			if ((!Boolean.valueOf(tab.getProperty("key")).booleanValue())
				&& (Boolean
					.valueOf(tab.getProperty("accessor"))
					.booleanValue())) {
				tab.accept(this);
				codePieces.add(codePiece);
			}
		}
	}
}
