/*
 * Created on 13-mrt-2004
 *
*/
package persistence.visitor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import persistence.Utilities;
import persistence.table.ArrayTable;
import persistence.table.Attribute;
import persistence.table.CollectionTable;
import persistence.table.MapTable;
import persistence.table.ReferenceAttribute;
import persistence.table.Table;
import persistence.table.TableComponent;

/**
 * Codegeneratie voor register-advice van queries.
 * @author bram
 *
 * 13-mrt-2004
 */
public class QueryRegisterVisitor implements TableComponentVisitor {
	private String fullClassName;
	private final String mem="mem";
	private final String thisType="thisType";
	private final String variableName="variableName";
	private final String realName="realName";
	private final String newAlias="newAlias";
	private ArrayList codePieces;
	private String codePiece;
	private ArrayList unimportantAttributes;
	private String tableName="";
	
	public QueryRegisterVisitor(String fullClassName){
		this.fullClassName=fullClassName;
		this.codePieces=new ArrayList();
		this.codePiece="";
		this.unimportantAttributes=new ArrayList();
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#getResult()
	 */
	public String getResult() {
		String res="pointcut register_"+fullClassName.replaceAll("\\.","_")+"(\n"+
		"MyMemoryQuery "+mem+",\n"+
		"String "+thisType+",\n"+
		"String "+variableName+") : this(\n"+
		mem+")\n"+
		"&& call(\n"+
			"private void com.sun.jdori.common.query.MyMemoryQuery.register(\n"+
				"String,\n"+
				"String) throws JDOQueryException)\n"+
		"&& args("+thisType+", "+variableName+")\n"+
		"&& if ("+thisType+".equals(\""+fullClassName+"\"));\n"+		
		"after(MyMemoryQuery "+mem+", String "+thisType+", String "+variableName+") throws JDOQueryException: register_"+fullClassName.replaceAll("\\.","_")+"(\n"+
				mem+",\n"+
				thisType+",\n"+
				variableName+") {\n"+				
				"String "+newAlias+" = \"alias\" + ("+mem+".aliasIndex++);\n"+
				"String "+realName+" = \"\";\n"+
				"int lastDot = "+variableName+".lastIndexOf(\".\");\n"+
				"if (lastDot != -1)\n"+
					realName+" = \"$\"+"+variableName+".substring(lastDot + 1)+\"$\";\n"+
				"else\n"+
					realName+" = "+variableName+"; //this/variable\n"+					
				"if ("+realName+".equals(\""+Utilities.fillIn(unimportantAttributes,"\")\n|| "+realName+".equals(\"",false)+"\")) {\n"+					
					"//doe niets (persistent wordt gedaan in this/variable van ander aspect)\n"+
				"}else ";
				
				Iterator it=codePieces.iterator();
				while(it.hasNext()){
					res+=(String)it.next();
				}
				
				return res+"}\n";
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitArrayTable(persistence.table.ArrayTable)
	 */
	public void visitArrayTable(ArrayTable ar) {
		codePiece="if ("+realName+".equals(\""+"$"+Utilities.extractAttributeName(ar.getComponentName())+"$"+"\")) {\n"+
			mem+".aliasToTableMapping.put("+newAlias+", \""+ar.getProperty("aliasName")+"\");\n"+
			mem+".nameToAliasMapping.put("+variableName+", "+newAlias+");\n}else ";
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitAttribute(persistence.table.Attribute)
	 */
	public void visitAttribute(Attribute attr) {
		unimportantAttributes.add(attr.getComponentName());
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitCollectionTable(persistence.table.CollectionTable)
	 */
	public void visitCollectionTable(CollectionTable coll) {
		codePiece="if ("+realName+".equals(\""+"$"+Utilities.extractAttributeName(coll.getComponentName())+"$"+"\")) {\n"+
					mem+".aliasToTableMapping.put("+newAlias+", \""+coll.getProperty("aliasName")+"\");\n"+
					mem+".nameToAliasMapping.put("+variableName+", "+newAlias+");\n}else ";					
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitMapTable(persistence.table.MapTable)
	 */
	public void visitMapTable(MapTable map) {
		// TODO: later?
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitReferenceAttribute(persistence.table.ReferenceAttribute)
	 */
	public void visitReferenceAttribute(ReferenceAttribute refAttr) {
		unimportantAttributes.add(refAttr.getComponentName());
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitTable(persistence.table.Table)
	 */
	public void visitTable(Table table) {
		tableName=table.getComponentName();
		Collection coll=table.getChildren();
		Iterator it=coll.iterator();
		while(it.hasNext()){
			codePiece="";//doorgegeven naar elk child
			TableComponent tab=(TableComponent)it.next();
			if((!Boolean.valueOf(tab.getProperty("key")).booleanValue())&&(Boolean.valueOf(tab.getProperty("accessor")).booleanValue())) tab.accept(this);
			else if(!Boolean.valueOf(tab.getProperty("accessor")).booleanValue()) codePiece+="if ("+realName+".equals(\""+tab.getComponentName()+"\")) { //no accessor!\n"+
						"throw new JDOQueryException(\"Attribute "+Utilities.extractAttributeName(tab.getComponentName())+" in type "+this.fullClassName+" is no accessor!\");\n"+			
					"}else ";
			codePieces.add(codePiece);
		}
		
		codePieces.add("{ //this/variable\n"+
		mem+".aliasToTableMapping.put("+newAlias+", \""+table.getProperty("aliasName")+"\");\n"+
		mem+".nameToAliasMapping.put("+variableName+", "+newAlias+");\n}\n");
	}
}
