/*
 * Created on 21-feb-2004
 *
*/
package persistence.visitor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import net.sf.hibernate.sql.Select;
import persistence.Utilities;
import persistence.table.ArrayTable;
import persistence.table.Attribute;
import persistence.table.CollectionTable;
import persistence.table.MapTable;
import persistence.table.ReferenceAttribute;
import persistence.table.Table;
import persistence.table.TableComponent;

/**
 * Creatie van read-advice van implementor-aspecten.
 * @author bram
 *
 * 21-feb-2004
 */
public class ReadVisitor implements TableComponentVisitor {
	private String fullClassName;
	private int nameDiscriminator = 0;
	//used for creating aliases: in stead of obj.getName().getFirstName(), one uses obj35.getFirstName()

	private ArrayList stack;
	/*contains ArrayLists with elements:	
		0: alias for previous Object
		1: alias for current Object
		2: alias for key of Map
		3: currentResultSetName
		4: keyMap
		*/
	private ArrayList resultStrings;
	//extra indentation, blocks, ... can be added in getresult() + declaration van stmt, ...(<-->transaction, ... in AbstractSQLFactoryImpl
	private Select currentSelect;
	private String currentAttribute;
	private String currentAlias;
	private String keyAlias;
	private String nextAlias;
	private String currentResultSetName;
	private Map keyMap;
	private String subTreeSQL; //SQL-code for subtree of main table
	private final String JDBCStatementName = "stmt";
	private final String JDBCPreparedStatementName = "prepStmt";
	private final String iteratorName = "it";
	private final String JDBCResultSetName = "res";
	private final String nameOfObject = "obj";
	private final String nameOfDeep = "deep";
	private final String nameOfOID = "OID";
	private String connectionExpression;

	public ReadVisitor(String fullClassName, String connectionExpression) {
		this.fullClassName = fullClassName;
		this.connectionExpression = connectionExpression;
		stack = new ArrayList();
		currentSelect = null; //initialized in visitTable
		currentAlias = nameOfObject;
		nextAlias = ""; //initialized in visitTable
		resultStrings = new ArrayList();
		currentResultSetName = ""; //initialized in visitTable
		keyMap = new HashMap();
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitAttribute(persistence.table.Attribute)
	 */
	public void visitAttribute(Attribute attr) {

		//$OID$, $ref...$ en $POSITION?$ niet bekijken (gebeurt in array/Map/Collection
		if (!(attr.getComponentName().matches("\\$POSITION\\d*\\$")
			|| attr.getComponentName().matches("\\$(ref)+.*\\$")
			|| attr.getComponentName().equals(
				"$OID$"))) { //||attr.getComponentName().equals("$KEY$")
			String nonQualifiedTypeName =
				Utilities.getUnqualifiedTypeName(attr.getJavaTypeName());
			subTreeSQL += attr.getJavaTypeName() + " " + nextAlias + "=";
			if (Utilities.isRealPrimitive(attr.getJavaTypeName())) {
				//echt primitief (geen wrapper), precies wat de get...-methoden teruggeven
				subTreeSQL += currentResultSetName
					+ "."
					+ Utilities.constructGetter(nonQualifiedTypeName, "dummy")
					+ "(\""
					+ attr.getComponentName()
					+ "\");\n";
			} else {
				if (attr.getJavaTypeName().equals("java.util.Date")) {
					//java.sql.Date teruggekregen
					subTreeSQL += "DateFormat.parse("
						+ currentResultSetName
						+ "."
						+ Utilities.constructGetter(
							nonQualifiedTypeName,
							"dummy")
						+ "(\""
						+ attr.getComponentName()
						+ "\").toString());\n";
				} else {
					if (nonQualifiedTypeName.equals("Integer")) {
						subTreeSQL += "new "
							+ nonQualifiedTypeName
							+ "("
							+ currentResultSetName
							+ "."
							+ Utilities.constructGetter("int", "dummy")
							+ "(\""
							+ attr.getComponentName()
							+ "\"));\n";
					} else {
						subTreeSQL += "new "
							+ nonQualifiedTypeName
							+ "("
							+ currentResultSetName
							+ "."
							+ Utilities.constructGetter(
								nonQualifiedTypeName,
								"dummy")
							+ "(\""
							+ attr.getComponentName()
							+ "\"));\n";
					}
				}
			}
		} else {
			if (attr.getComponentName().equals("$OID$")) {
				subTreeSQL += "persistence.type.ObjectIDFactory fac"
					+ nextAlias
					+ "=new persistence.type.ObjectIDFactory();\n"
					+ "persistence.type.ObjectID "
					+ nextAlias
					+ "=fac"
					+ nextAlias
					+ ".translateToObjectID("
					+ nameOfOID
					+ ".toString()"
					+ ");\n";
			}
		}

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitReferenceAttribute(persistence.table.ReferenceAttribute)
	 */
	public void visitReferenceAttribute(ReferenceAttribute refAttr) {
		subTreeSQL += "persistence.type.ObjectIDFactory fac"
			+ nextAlias
			+ "=new persistence.type.ObjectIDFactory();\n"
			+ refAttr.getProperty("referencedJavaTypeName")
			+ " "
			+ nextAlias
			+ "=null;\n"
			+ "if(!"
			+ currentResultSetName
			+ ".getString(\""
			+ refAttr.getComponentName()
			+ "\").equalsIgnoreCase(\"null\")){\n"
			+ "if(!"
			+ nameOfDeep
			+ "){\n"
			+ nextAlias
			+ "=("
			+ refAttr.getProperty("referencedJavaTypeName")
			+ ")persistence.Utilities.constructObject(fac"
			+ nextAlias
			+ ".translateToObjectID("
			+ currentResultSetName
			+ ".getString(\""
			+ refAttr.getComponentName()
			+ "\")));\n"
			+ nextAlias
			+ ".setOID(fac"
			+ nextAlias
			+ ".translateToObjectID("
			+ currentResultSetName
			+ ".getString(\""
			+ refAttr.getComponentName()
			+ "\")));\n"
			+ "}else{\n"
			+ nextAlias
			+ "=("
			+ refAttr.getProperty("referencedJavaTypeName")
			+ ")persistence.type.Persistent.QueryManager.read(fac"
			+ nextAlias
			+ ".translateToObjectID("
			+ currentResultSetName
			+ ".getString(\""
			+ refAttr.getComponentName()
			+ "\")),"
			+ nameOfDeep
			+ ");\n"
			+ "}}\n";
	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitTable(persistence.table.Table)
	 */
	public void visitTable(Table table) {
		String extend = table.getProperty("extends");

		keyMap.put("$OID$", "'\"+" + nameOfOID + ".toString()+\"'");
		currentResultSetName = JDBCResultSetName;
		currentSelect = new Select();
		currentSelect.setOuterJoins("", "");
		currentSelect.setFromClause(table.getProperty("aliasName"));
		currentSelect.setSelectClause("*");
		currentSelect.setWhereClause("$OID$=" + (String) keyMap.get("$OID$"));
		String res =
			JDBCStatementName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n"
				+ currentResultSetName
				+ "="
				+ JDBCStatementName
				+ ".executeQuery(\""
				+ currentSelect.toStatementString()
				+ "\");\n";

		res += "Object " + nameOfObject + "=null;\n";
		res += "if(" + currentResultSetName + ".next()){\n";
		//als geen rij aanwezig, dan null!!!

		//		als child van persistent parent, dan nep-OID maken voor parent, die lezen en daarop voort bouwen
		if (Boolean
			.valueOf(table.getProperty("persistentChild"))
			.booleanValue())
			res += "ObjectID tmp="
				+ nameOfOID
				+ ";\n"
				+ "tmp.setCClass(\""
				+ extend
				+ "\");\n"
				+ nameOfObject
				+ "=("
				+ table.getJavaTypeName()
				+ ")Persistent.QueryManager.read(tmp,"
				+ nameOfDeep
				+ ");\n";
		else //eindpunt van inheritance tree					
			res += currentAlias
				+ "=persistence.Utilities.constructObject("
				+ nameOfOID
				+ ");\n";
		//currentAlias==nameOfObject
		resultStrings.add(res);

		List children = table.getChildren();
		Iterator it = children.iterator();

		while (it.hasNext()) {
			subTreeSQL = new String("");
			/*
			 * subTreeSQL will be passed to deeper level as long as there are Maps/Collections/Arrays.
			 */
			TableComponent tab = (TableComponent) it.next();
			nextAlias = currentAlias + (nameDiscriminator++);
			//naam van gecreerd object in children
			tab.accept(this);
			resultStrings.add(
				subTreeSQL
					+ "(("
					+ table.getJavaTypeName()
					+ ")"
					+ currentAlias
					+ ")."
					+ tab.getProperty("set")
					+ "("
					+ nextAlias
					+ ");\n");
		}
			res = "}\n" + //einde van 1 else (resultSet.hasNext)
	currentResultSetName + ".close();\n" + JDBCStatementName + ".close();\n";
		resultStrings.add(res);

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitArrayTable(persistence.table.ArrayTable)
	 */
	public void visitArrayTable(ArrayTable ar) {
		Vector stackEntry = new Vector();
		stackEntry.add(currentAlias);
		stackEntry.add(nextAlias);
		stackEntry.add(currentResultSetName);
		stackEntry.add(keyMap);
		stack.add(stackEntry);

		//new state
		keyMap = Utilities.referize(keyMap);
		int index = (nameDiscriminator++);
		//plakken aan Statement en ResultSet + close() beide na de while
		currentAlias = nextAlias;
		nextAlias = nameOfObject + index;
		//slechts 1 attribuut/Map/Collection zal het gebruiken
		currentResultSetName = JDBCResultSetName + index;
		String currentStmtName = JDBCStatementName + index;

		currentSelect = new Select();
		currentSelect.setOuterJoins("", "");
		currentSelect.setFromClause(ar.getProperty("aliasName"));

		//		creer where-clause
		currentSelect.setWhereClause(createWhereClause());
		int dim = (new Integer(ar.getProperty("dimension"))).intValue();
		String tmp =
			subTreeSQL
				+ "Statement "
				+ currentStmtName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n";
		//eerst dimensies opvragen
		String selectClause = "";
		for (int i = 0; i < dim; i++) {
			selectClause += "max($POSITION"
				+ (i + 1)
				+ "$) as aantal"
				+ (i + 1)
				+ ",";
		}
		currentSelect.setSelectClause(
			selectClause.substring(0, selectClause.length() - 1));
		//laatste komma weg

		tmp += "ResultSet "
			+ currentResultSetName
			+ "="
			+ currentStmtName
			+ ".executeQuery(\""
			+ currentSelect.toStatementString()
			+ "\");\n"
			+ ar.getJavaTypeName()
			+ Utilities.constructArrayHaakjes(dim, ' ')
			+ " "
			+ currentAlias
			+ "=null;\n";
		//tijdelijk opslaan om toevoegen "if" eleganter te maken
		String ifNullOpen = "if(" + currentResultSetName + ".next()){\n";
		//als geen rij aanwezig, dan null!!!
		String ifNullClosed = "}\n";

		String creatieHaakjes = "";
		String iteratieHaakjes = "";
		for (int i = 1; i <= dim; i++) {
			creatieHaakjes += "[("
				+ currentResultSetName
				+ ".getInt(\"aantal"
				+ i
				+ "\")"
				+ "+1)]";
			iteratieHaakjes += "["
				+ currentResultSetName
				+ ".getInt(\"$POSITION"
				+ i
				+ "$\")"
				+ "]";
			keyMap.put(
				"$POSITION" + i + "$",
				"\"+"
					+ currentResultSetName
					+ ".getInt(\"$POSITION"
					+ i
					+ "$\")+\"");
		}

		//		hervul subTreeSQL
		subTreeSQL =
			currentAlias
				+ "= new "
				+ ar.getJavaTypeName()
				+ creatieHaakjes
				+ ";\n";

		//nu inhoud opvragen
		currentSelect.setSelectClause("*");
		subTreeSQL += currentResultSetName
			+ "="
			+ currentStmtName
			+ ".executeQuery(\""
			+ currentSelect.toStatementString()
			+ "\");\n"
			+ currentResultSetName
			+ ".next();\n"
			+ "do{\n";

		List children = ar.getChildren();
		Iterator it = children.iterator();
		//		inlezen object nextAlias
		while (it.hasNext()) {
			TableComponent tab = (TableComponent) it.next();
			//al gezet (er is slechts 1 nodig attribuut) nextAlias=currentAlias+(nameDiscriminator++);//naam van gecreerd object in children
			tab.accept(this);
		}
		subTreeSQL += currentAlias
			+ iteratieHaakjes
			+ "="
			+ nextAlias
			+ ";\n"
			+ "}while("
			+ currentResultSetName
			+ ".next());\n";
		//einde while

		subTreeSQL =
			tmp
				+ ifNullOpen
				+ subTreeSQL
				+ ifNullClosed
				+ currentResultSetName
				+ ".close();\n"
				+ currentStmtName
				+ ".close();\n";

		stackEntry = (Vector) stack.remove(stack.size() - 1);
		currentAlias = (String) stackEntry.get(0);
		nextAlias = (String) stackEntry.get(1);
		currentResultSetName = (String) stackEntry.get(2);
		keyMap = (Map) stackEntry.get(3);

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitCollectionTable(persistence.table.CollectionTable)
	 */
	public void visitCollectionTable(CollectionTable coll) {
		Vector stackEntry = new Vector();
		stackEntry.add(currentAlias);
		stackEntry.add(nextAlias);
		stackEntry.add(currentResultSetName);
		stackEntry.add(keyMap);
		stack.add(stackEntry);

		//new state
		keyMap = Utilities.referize(keyMap);
		int index = (nameDiscriminator++);
		//plakken aan Statement en ResultSet + close() beide na de while
		currentAlias = nextAlias;
		nextAlias = nameOfObject + index;
		//slechts 1 attribuut/Map/rij zal het gebruiken
		currentResultSetName = JDBCResultSetName + index;
		//currentOIDName=nextOIDName;//dit object is al ingelezen, daar $(ref)OID$ eerst ingelezen wordt
		String currentStmtName = JDBCStatementName + index;

		currentSelect = new Select();
		currentSelect.setOuterJoins("", "");
		currentSelect.setFromClause(coll.getProperty("aliasName"));
		currentSelect.setWhereClause(createWhereClause());

		String tmp =
			subTreeSQL
				+ "Statement "
				+ currentStmtName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n";
		//tijdelijk opslaan om toevoegen "if" eleganter te maken
		String impClass = (String) coll.getProperty("impClass");
		//nog niet nodig in ifNullOpen omdat daar geen object gecreerd wordt
		String ifNullOpen = "if(" + currentResultSetName + ".next()){\n";
		//als geen rij aanwezig, dan null!!!
		String ifNullClosed = "}\n";

		keyMap.put(
			"$POSITION$",
			"\"+" + currentResultSetName + ".getInt(\"$POSITION$\")+\"");

		if (impClass == null) {
			subTreeSQL =
				currentAlias + "= new " + coll.getJavaTypeName() + "();\n";
		} else {
			subTreeSQL = currentAlias + "= new " + impClass + "();\n";
		}

		//nu inhoud opvragen
		currentSelect.setSelectClause("*");
		tmp += "ResultSet "
			+ currentResultSetName
			+ "="
			+ currentStmtName
			+ ".executeQuery(\""
			+ currentSelect.toStatementString()
			+ "\");\n"
			+ coll.getJavaTypeName()
			+ " "
			+ currentAlias
			+ "=null;\n";
		subTreeSQL += "do{\n";

		List children = coll.getChildren();
		Iterator it = children.iterator();
		//		inlezen object nextAlias
		while (it.hasNext()) {

			TableComponent tab = (TableComponent) it.next();
			tab.accept(this);
		}
		subTreeSQL += currentAlias
			+ ".add("
			+ nextAlias
			+ ");\n"
			+ "}while("
			+ currentResultSetName
			+ ".next());\n";
		//einde while

		subTreeSQL =
			tmp
				+ ifNullOpen
				+ subTreeSQL
				+ ifNullClosed
				+ currentResultSetName
				+ ".close();\n"
				+ currentStmtName
				+ ".close();\n";

		stackEntry = (Vector) stack.remove(stack.size() - 1);
		currentAlias = (String) stackEntry.get(0);
		nextAlias = (String) stackEntry.get(1);
		currentResultSetName = (String) stackEntry.get(2);
		keyMap = (Map) stackEntry.get(3);

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#visitMapTable(persistence.table.MapTable)
	 */
	public void visitMapTable(MapTable map) {
		Vector stackEntry = new Vector();
		stackEntry.add(currentAlias);
		stackEntry.add(nextAlias);
		stackEntry.add(currentResultSetName);
		stackEntry.add(keyMap);
		stack.add(stackEntry);

		//new state
		keyMap = Utilities.referize(keyMap);
		int index = (nameDiscriminator++);
		//plakken aan Statement en ResultSet + close() beide na de while
		currentAlias = nextAlias;

		currentResultSetName = JDBCResultSetName + index;
		String currentStmtName = JDBCStatementName + index;

		currentSelect = new Select();
		currentSelect.setOuterJoins("", "");
		currentSelect.setFromClause(map.getProperty("aliasName"));
		currentSelect.setWhereClause(createWhereClause());

		String tmp =
			subTreeSQL
				+ "Statement "
				+ currentStmtName
				+ "="
				+ connectionExpression
				+ ".createStatement();\n";

		//tijdelijk opslaan om toevoegen "if" eleganter te maken
		String impClass = (String) map.getProperty("impClass");
		//nog niet nodig in ifNullOpen omdat daar geen object gecreerd wordt
		String ifNullOpen = "if(" + currentResultSetName + ".next()){\n";
		//als geen rij aanwezig, dan null!!!

		String ifNullClosed = "}\n";
		String nonQualifiedTypeName =
			Utilities.getUnqualifiedTypeName(map.getProperty("compTypeKey"));
		if (nonQualifiedTypeName.equals("String")) {
			keyMap.put(
				"$KEY$",
				"'\"+"
					+ currentResultSetName
					+ "."
					+ Utilities.constructGetter(nonQualifiedTypeName, "dummy")
					+ "(\"$KEY$\")+\"'");
		} else if (Utilities.isPrimitive(nonQualifiedTypeName)) {
			keyMap.put(
				"$KEY$",
				"\"+"
					+ currentResultSetName
					+ "."
					+ Utilities.constructGetter(nonQualifiedTypeName, "dummy")
					+ "(\"$KEY$\")+\"");
		} else { //persistent object
			keyMap.put(
				"$KEY$",
				"'\"+" + currentResultSetName + ".getString(\"$KEY$\")+\"'");
		}

		//	  check impClass en hervul subTreeSQL	  
		if (impClass == null) {
			subTreeSQL =
				currentAlias + "= new " + map.getJavaTypeName() + "();\n";
		} else {
			subTreeSQL = currentAlias + "= new " + impClass + "();\n";
		}

		//nu inhoud opvragen
		currentSelect.setSelectClause("*");
		tmp += "ResultSet "
			+ currentResultSetName
			+ "="
			+ currentStmtName
			+ ".executeQuery(\""
			+ currentSelect.toStatementString()
			+ "\");\n"
			+ map.getJavaTypeName()
			+ " "
			+ currentAlias
			+ "=null;\n";
		subTreeSQL += "do{\n";

		List children = map.getChildren();
		Iterator it = children.iterator();
		//	  inlezen object nextAlias
		Vector vec = new Vector();
		while (it.hasNext()) {
			TableComponent tab = (TableComponent) it.next();
			if (!tab.getComponentName().matches("\\$(ref)+.*\\$")) {
				nextAlias = nameOfObject + (nameDiscriminator++);
				//key en value
				vec.add(nextAlias);
			}
			tab.accept(this);
		}
		if (Utilities.isPrimitive(map.getProperty("compTypeKey"))) {
			subTreeSQL += currentAlias
				+ ".put("
				+ (String) vec.get(0)
				+ ","
				+ (String) vec.get(1)
				+ ");\n"
				+ "}while("
				+ currentResultSetName
				+ ".next());\n";
			//einde while
		} else { //persistent object: key kan null zijn => verhinderen
			subTreeSQL += "if("
				+ (String) vec.get(0)
				+ "!=null) "
				+ currentAlias
				+ ".put("
				+ (String) vec.get(0)
				+ ","
				+ (String) vec.get(1)
				+ ");\n"
				+ "}while("
				+ currentResultSetName
				+ ".next());\n";
			//einde while
		}

		subTreeSQL =
			tmp
				+ ifNullOpen
				+ subTreeSQL
				+ ifNullClosed
				+ currentResultSetName
				+ ".close();\n"
				+ currentStmtName
				+ ".close();\n";

		stackEntry = (Vector) stack.remove(stack.size() - 1);
		currentAlias = (String) stackEntry.get(0);
		nextAlias = (String) stackEntry.get(1);
		currentResultSetName = (String) stackEntry.get(2);
		keyMap = (Map) stackEntry.get(3);

	}

	/**
	 * @see persistence.visitor.TableComponentVisitor#getResult()
	 */
	public String getResult() {
		String res =
			"pointcut reader_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "(ObjectID "
				+ nameOfOID
				+ ",boolean "
				+ nameOfDeep
				+ "): args("
				+ nameOfOID
				+ ","
				+ nameOfDeep
				+ ") && execution(public Object persistence.type.Persistent.QueryManager.read(ObjectID,boolean)) && if("
				+ nameOfOID
				+ ".getCClass().equals(\""
				+ Utilities.checkClassName(fullClassName)
				+ "\"));\n"
				+ "Object around(ObjectID "
				+ nameOfOID
				+ ",boolean "
				+ nameOfDeep
				+ "): reader_"
				+ fullClassName.replaceAll("\\.", "_")
				+ "("
				+ nameOfOID
				+ ","
				+ nameOfDeep
				+ "){\n"
				+ "ConnectionFactory conFac=new ConnectionFactory();\n"
				+ "Connection "
				+ connectionExpression
				+ "=conFac.getConnection();\n"
				+ "try{\n"
				+ "Statement "
				+ JDBCStatementName
				+ "=null;\n"
				+ "Iterator "
				+ iteratorName
				+ "=null;\n"
				+ "ResultSet "
				+ JDBCResultSetName
				+ "=null;\n";
		for (int i = 0; i < resultStrings.size(); i++) {
			res = res + resultStrings.get(i);
		}
		res += "return "
			+ nameOfObject
			+ ";\n"
			+ "}catch(Exception e){\n"
			+ "//doe voorlopig niets en gebruik transactie-aspect\n"
			+ "}finally{\n"
			+ "conFac.closeConnection("
			+ connectionExpression
			+ ");\n"
			+ "}\n"
			+ "return null;\n";
		return res + "}\n";
	}

	/**
	 * Create where-clause with all the $ref...$-fields.
	 * @return
	 */
	private String createWhereClause() {
		String res = "";
		Iterator it = keyMap.keySet().iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			String value = (String) keyMap.get(key);
			res += key + "=" + value + " AND ";
		}
		return res.substring(0, res.length() - 5);
	}

}
