/*
 * Created on 12-apr-2004
 *
*/
package tests;

import Script;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Vector;

import persistence.exceptions.UpdateException;
import persistence.transaction.Transaction;
import persistence.type.Persistent;

import junit.framework.TestCase;
import casestudy.Address;
import casestudy.Course;
import casestudy.Curriculum;
import casestudy.Major;
import casestudy.Name;
import casestudy.Student;

/**
 * @author bram
 *
 * 12-apr-2004
 */
public class TransactionTest extends TestCase {
	private Course cou=null;

	/**
	 * Constructor for TransactionTest.
	 * @param arg0
	 */
	public TransactionTest(String arg0) {
		super(arg0);
	}
	
	public void testCommit(){
		Transaction trans=Persistent.QueryManager.newTransaction();
		trans.start();
		Course read=(Course)Persistent.QueryManager.read(trans,cou.getOID(),true);
		assertEquals("Programmeren II: C en C++",read.getCourseName());
		read.setCourseName("Programmeren in C#");
		try{
		read.update(trans);
	}catch(UpdateException ex){
		ex.printStackTrace();			
		trans.rollback();
		return;
	}
		read=(Course)Persistent.QueryManager.read(trans,cou.getOID(),false);
		assertEquals("Programmeren in C#",read.getCourseName());
		trans.commit();
	}
	
	public void testRollback(){
		Transaction trans=Persistent.QueryManager.newTransaction();
		trans.start();
		Course read=(Course)Persistent.QueryManager.read(trans,cou.getOID(),false);
		assertEquals("Programmeren II: C en C++",read.getCourseName());
		read.setCourseName("Programmeren in C#");
		try{
			read.update(trans);
		}catch(UpdateException ex){
			ex.printStackTrace();			
			trans.rollback();
			return;
		}
		read=(Course)Persistent.QueryManager.read(trans,cou.getOID(),true);
		assertEquals("Programmeren in C#",read.getCourseName());
		trans.rollback();
		read=(Course)Persistent.QueryManager.read(trans,cou.getOID(),false);
		assertEquals("Programmeren II: C en C++",read.getCourseName());
	}
	
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();
		Script.main(null);//drop and create tables
		cou = new Course();
		cou.setCourseID("ANALY1");
		cou.setCourseName("Analyse I");
		cou.setCourseWeight(10);
		Course analysOne = cou;
		cou.write();

		cou = new Course();
		cou.setCourseID("ANALY2");
		cou.setCourseName("Analyse II");
		cou.setCourseWeight(8);
		Collection coll = new Vector();
		coll.add(analysOne);
		cou.setPrerequisites(coll);
		Course analysTwo = cou;
		cou.write();
		
		cou = new Course();
		cou.setCourseID("ANALY3");
		cou.setCourseName("Analyse III");
		cou.setCourseWeight(8);
		coll = new Vector();
		coll.add(analysOne);
		coll.add(analysTwo);
		cou.setPrerequisites(coll);
		Course analysThree = cou;
		cou.write();
		
		cou = new Course();
		cou.setCourseID("PROGR1");
		cou.setCourseName("Programmeren I: Java");
		cou.setCourseWeight(6);		
		Course progrOne = cou;
		cou.write();
		
		cou = new Course();
		cou.setCourseID("PROGR2");
		cou.setCourseName("Programmeren II: C en C++");
		cou.setCourseWeight(7);
		coll = new Vector();
		coll.add(progrOne);		
		cou.setPrerequisites(coll);
		Course progrTwo = cou;
		cou.write();
		
		Student stu=new Student();
		stu.setStudentID("19991539");
		Address adres=new Address();
		adres.setCity("Waregem");
		adres.setNumber(9);
		adres.setStreet("Roterijstraat");
		adres.setZip("8790");
		stu.setAdres(adres);
		Name name=new Name();
		name.setFirstName("Bram");
		name.setLastName("Adams");
		stu.setName(name);
		Major maj = new Major();
		maj.setDepartment("Science Department");
		maj.setFaculty("Faculty of Applied Sciences");
		maj.setMajorID("Comp");
				maj.write();
				stu.setYearOfStudy(maj);
				Curriculum cur=new Curriculum();
				cur.setLow(10);
				cur.setHigh(20);
				coll=new ArrayList();
				coll.add(analysOne);
				coll.add(analysTwo);
				coll.add(analysThree);
				coll.add(progrOne);
				coll.add(progrTwo);
				cur.setCourses(coll);
				cur.write();
				stu.setProgramOfStudy(cur);
				stu.write();
				
		stu=new Student();
				stu.setStudentID("19971433");
				adres=new Address();
				adres.setCity("Durbuy");
				adres.setNumber(64);
				adres.setStreet("Rue de Comte Thodore");
				adres.setZip("6490");
				stu.setAdres(adres);
				name=new Name();
				name.setFirstName("Andr");
				name.setLastName("Hpital");
				stu.setName(name);
				maj = new Major();
		maj.setDepartment("Science Department");
		maj.setFaculty("Faculty of Sciences");
		maj.setMajorID("Math");

						maj.write();
						stu.setYearOfStudy(maj);
						cur=new Curriculum();
						cur.setLow(5);
						cur.setHigh(16);
						coll=new ArrayList();
						coll.add(analysOne);
						coll.add(analysTwo);
						cur.setCourses(coll);
						cur.write();
						stu.setProgramOfStudy(cur);
						stu.write();
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();
	}

}
