/*
 * Copyright 2001 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */

/*
 * JDOQL.g
 *
 * Created on August 28, 2001
 */

header
{
    package com.sun.jdori.common.query.jdoqlc;

    import antlr.MismatchedTokenException;
    import antlr.MismatchedCharException;
    import antlr.NoViableAltException;
    import antlr.NoViableAltForCharException;
    import antlr.TokenStreamRecognitionException;
    
    import javax.jdo.I18NHelper;

    import com.sun.jdori.query.JDOQueryException;
    import com.sun.jdori.common.query.tree.*;
}

//===== Lexical Analyzer Class Definitions =====

/**
 * This class defines the lexical analysis for the JDOQL compiler.
 *
 * @author  Michael Bouschen
 */
class JDOQLLexer extends Lexer;
options
{
    k = 2;
    exportVocab = JDOQL;
    charVocabulary = '\3'..'\uffef'; //NOI18N
}

tokens {

    IMPORT = "import"         <AST=com.sun.jdori.common.query.tree.Node>; //NOI18N
    IMPORT_ON_DEMAND          <AST=com.sun.jdori.common.query.tree.Node>;
    THIS = "this"             <AST=com.sun.jdori.common.query.tree.ThisExpr>; //NOI18N
    ASCENDING = "ascending"   <AST=com.sun.jdori.common.query.tree.AscendingOrderingExpr>; //NOI18N
    DESCENDING = "descending" <AST=com.sun.jdori.common.query.tree.DescendingOrderingExpr>; //NOI18N
    
    // types
    
    BOOLEAN = "boolean"       <AST=com.sun.jdori.common.query.tree.Node>; //NOI18N
    BYTE = "byte"             <AST=com.sun.jdori.common.query.tree.Node>; //NOI18N
    CHAR = "char"             <AST=com.sun.jdori.common.query.tree.Node>; //NOI18N
    SHORT = "short"           <AST=com.sun.jdori.common.query.tree.Node>; //NOI18N
    INT = "int"               <AST=com.sun.jdori.common.query.tree.Node>; //NOI18N
    FLOAT = "float"           <AST=com.sun.jdori.common.query.tree.Node>; //NOI18N
    LONG = "long"             <AST=com.sun.jdori.common.query.tree.Node>; //NOI18N
    DOUBLE = "double"         <AST=com.sun.jdori.common.query.tree.Node>; //NOI18N
    
    // literals
    
    NULL = "null"   <AST=com.sun.jdori.common.query.tree.ConstantExpr>; //NOI18N
    TRUE = "true"   <AST=com.sun.jdori.common.query.tree.BooleanLiteralExpr>; //NOI18N
    FALSE = "false" <AST=com.sun.jdori.common.query.tree.BooleanLiteralExpr>; //NOI18N

    // other tokens

    EQUAL     <AST=com.sun.jdori.common.query.tree.EqualsExpr>;
    NOT_EQUAL <AST=com.sun.jdori.common.query.tree.NotEqualsExpr>;
    GE        <AST=com.sun.jdori.common.query.tree.GreaterThanEqualsExpr>;
    GT        <AST=com.sun.jdori.common.query.tree.GreaterThanExpr>;
    LE        <AST=com.sun.jdori.common.query.tree.LessThanEqualsExpr>;
    LT        <AST=com.sun.jdori.common.query.tree.LessThanExpr>;

    PLUS      <AST=com.sun.jdori.common.query.tree.PlusExpr>;
    MINUS     <AST=com.sun.jdori.common.query.tree.MinusExpr>;
    STAR      <AST=com.sun.jdori.common.query.tree.TimesExpr>;
    DIV       <AST=com.sun.jdori.common.query.tree.DivideExpr>;

    BOR       <AST=com.sun.jdori.common.query.tree.OrExpr>;
    OR        <AST=com.sun.jdori.common.query.tree.ConditionalOrExpr>;
    BAND      <AST=com.sun.jdori.common.query.tree.AndExpr>;
    AND       <AST=com.sun.jdori.common.query.tree.ConditionalAndExpr>;
    LNOT      <AST=com.sun.jdori.common.query.tree.NotExpr>;
    BNOT      <AST=com.sun.jdori.common.query.tree.ComplementExpr>;

    INT_LITERAL    <AST=com.sun.jdori.common.query.tree.IntLiteralExpr>;
    LONG_LITERAL   <AST=com.sun.jdori.common.query.tree.IntLiteralExpr>;
    FLOAT_LITERAL  <AST=com.sun.jdori.common.query.tree.FloatLiteralExpr>;
    DOUBLE_LITERAL <AST=com.sun.jdori.common.query.tree.DoubleLiteralExpr>;
    CHAR_LITERAL   <AST=com.sun.jdori.common.query.tree.CharLiteralExpr>;
    STRING_LITERAL <AST=com.sun.jdori.common.query.tree.ConstantExpr>;
    IDENT          <AST=com.sun.jdori.common.query.tree.IdentifierExpr>;

    DOT <AST=com.sun.jdori.common.query.tree.Node>;

    // lexer only

    LPAREN;
    RPAREN;
    COMMA;
    SEMI;
    WS;
    NEWLINE;
    ESC; 
    HEX_DIGIT;
    EXPONENT;
    FLOAT_SUFFIX;
    DOUBLE_SUFFIX;
    UNICODE_DIGIT;
    UNICODE_STR;
}

{
    /** The error message support class. */
    protected ErrorMsg errorMsg;
    
    /** Start column of the actual token. */
    protected int tokColumn = 1;

    /** The actual column. */
    protected int column = 1;

    /**
     * The width of a tab stop.
     * This value is used to calculate the correct column in a line
     * conatining a tab character.
     */
    protected int TABSIZE = 4;

    /** I18N support. */
    protected final static I18NHelper msg = 
        I18NHelper.getInstance("com.sun.jdori.common.query.Bundle"); //NOI18N
    
    /**
     *
     */
    public void init(ErrorMsg errorMsg)
    {
        this.errorMsg = errorMsg;
    }
    
    /**
     * Redefinition of consume method for tracking column information
     */
    public void consume() throws CharStreamException
    {
        if (text.length()==0)
        {
            // remember token start column
            tokColumn = column;
        }
        char lookahead = LA(1);
        switch (lookahead)
        {
        case '\n':
            column = 1;
            break;
        case '\t':
            int mod = column % TABSIZE;
            if ( mod != 0 ) column += (TABSIZE - mod);
            column++;
            break;
        default:
            column++;
            break;
        }
        super.consume();
    }
    
    /**
     * Redefinition of makeToken method to set the token column start value for the created tokens.
     */
    protected Token makeToken(int t)
    {
        Token tok = super.makeToken(t);
        tok.setColumn(tokColumn);
        return tok;
    }
    
    /**
     * Token class for the JDOQLLexer.
     * Adds column information to the ANTLR token class.
     */
    public static class JDOQLToken extends antlr.CommonToken
    {
        public JDOQLToken()
        {
            super();
        }
        public JDOQLToken(int type, String text) 
        {
            super(type, text);
        }
        protected int column;
        public int getColumn() { return column; }
        public void setColumn(int c) { column = c; }
    }

    /**
     *
     */
    public void reportError(int line, int column, String s)
    {
        errorMsg.error(line, column, s);
    }

    /**
     * Report lexer exception errors caught in nextToken()
     */
    public void reportError(RecognitionException e)
    {
        handleANTLRException(e, errorMsg);
    }

    /**
     * Lexer error-reporting function
     */
    public void reportError(String s)
    {
        errorMsg.error(0, 0, s);
    }

    /**
     * Lexer warning-reporting function
     */
    public void reportWarning(String s)
    {
        throw new JDOQueryException(s);
    }

    /**
     *
     */
    public static void handleANTLRException(ANTLRException ex, ErrorMsg errorMsg)
    {
        if (ex instanceof MismatchedCharException) {
            MismatchedCharException mismatched = (MismatchedCharException)ex;
            if (mismatched.mismatchType == MismatchedCharException.CHAR) {
                errorMsg.error(mismatched.getLine(), mismatched.getColumn(), 
                    msg.msg("EXC_ExpectedCharFound", //NOI18N
                        String.valueOf((char)mismatched.expecting), 
                        String.valueOf(mismatched.foundChar)));
                return;
            }
        }
        else if (ex instanceof MismatchedTokenException) {
            JDOQLLexer.JDOQLToken token = 
                (JDOQLLexer.JDOQLToken) ((MismatchedTokenException)ex).token;
            if (token != null) {
                errorMsg.error(token.getLine(), token.getColumn(), 
                    msg.msg("EXC_SyntaxErrorAt", token.getText())); //NOI18N
                return;
            }
        }
        else if (ex instanceof NoViableAltException) {
            JDOQLLexer.JDOQLToken token = (JDOQLLexer.JDOQLToken) ((NoViableAltException)ex).token;
            if (token != null) {
                errorMsg.error(token.getLine(), token.getColumn(), 
                               msg.msg("EXC_UnexpectedToken", token.getText())); //NOI18N
                return;
            }
        }
        else if (ex instanceof NoViableAltForCharException) {
            NoViableAltForCharException noViableAlt = (NoViableAltForCharException)ex;
            errorMsg.error(noViableAlt.getLine(), noViableAlt.getColumn(), 
                msg.msg("EXC_UnexpectedChar", String.valueOf(noViableAlt.foundChar)));//NOI18N
        }
        else if (ex instanceof TokenStreamRecognitionException) {
            handleANTLRException(((TokenStreamRecognitionException)ex).recog, errorMsg);
        }

        // no special handling from aboves matches the exception if this line is reached =>
        // make it a syntax error
        int line = 0;
        int column = 0;
        if (ex instanceof RecognitionException) {
            line = ((RecognitionException)ex).getLine();
            column = ((RecognitionException)ex).getColumn();
        }
        errorMsg.error(line, column, msg.msg("EXC_SyntaxError")); //NOI18N
    }
}

// OPERATORS

// The following Java operators are included although not supported by JDOQL:
//   ASSIGN, DIV_ASSIGN, PLUS_ASSIGN, INC, MINUS_ASSIGN, DEC, STAR_ASSIGN, 
//   SR, SL, BOR_ASSIGN, BAND_ASSIGN
// The lexer recognizes them, but the parser will result in a syntax error.
// This gurantees an error message. Othewise the java operator pre-decrement
// is mapped to two unray minus. Two unray minus are valid in JDOQL, but not 
// indented.

LPAREN          :   '('     ;
RPAREN          :   ')'     ;
COMMA           :   ','     ;
//DOT           :   '.'     ;
ASSIGN          :   "="     ; //NOI18N
EQUAL           :   "=="    ; //NOI18N
LNOT            :   '!'     ;
BNOT            :   '~'     ;
NOT_EQUAL       :   "!="    ; //NOI18N
DIV             :   '/'     ;
DIV_ASSIGN      :   "/="    ; //NOI18N
PLUS            :   '+'     ;
PLUS_ASSIGN     :   "+="    ; //NOI18N
INC             :   "++"    ; //NOI18N
MINUS           :   '-'     ;
MINUS_ASSIGN    :   "-="    ; //NOI18N
DEC             :   "--"    ; //NOI18N
STAR            :   '*'     ;
STAR_ASSIGN     :   "*="    ; //NOI18N
SR              :   ">>"    ; //NOI18N
GE              :   ">="    ; //NOI18N
GT              :   ">"     ; //NOI18N
SL              :   "<<"    ; //NOI18N
LE              :   "<="    ; //NOI18N
LT              :   '<'     ;
BOR             :   '|'     ;
BOR_ASSIGN      :   "|="    ; //NOI18N
OR              :   "||"    ; //NOI18N
BAND            :   '&'     ;
BAND_ASSIGN     :   "&="    ; //NOI18N
AND             :   "&&"    ; //NOI18N
SEMI            :   ';'     ;

// Whitespace -- ignored
// MBO: 
// I use different rules for WS and NEWLINE to prevent that antlr generates
// another bitset. Otherwise both rules could be combioned to a single rule WS.
WS
    :   (   ' '
        |   '\t'
        |   '\f'
        )
        { _ttype = Token.SKIP; }
    ;

NEWLINE
    :   (   "\r\n"  //NOI18N
        |   '\r'
        |   '\n'
        )
        { 
            newline(); 
            _ttype = Token.SKIP; 
        }
    ;

// MBO: 
// The common rules to scan char and string literals are:
//   CHAR_LITERAL :  '\'' ( ESC | ~'\'' ) '\'' ;
//   STRING_LITERAL : '"' ( ESC | ~'"')* '"' ;
// They do not work, because
// - For matching ESC antlr-2.7.0 generates a switch having unicode constants
//   as case labels. In the case of a uncode constant \u0xxx antlr-2.7.0 does 
//   not generate the leading 0 digit which leads to syntax errors in the 
//   generated java file. This is a know bug which is fxied in antlr-2.7.1. 
//   I add a semantic rule checking the first char beeing a \ as a workaround.
// - The ~ rule leads to another big bitset. I replaced it by a rule matching
//   any char plus a semantic rule excluding the end token char 
//   (single or double quote).

// character literals
CHAR_LITERAL
    :   '\'' 
        (   { LA(1)=='\\'}? ESC 
        |   { LA(1)!='\''}? . 
        ) 
        '\''
    ;

// string literals
STRING_LITERAL
    :  '"' ({ LA(1)=='\\' }? ESC | { LA(1) != '"' }? . )* '"' //NOI18N
    ;

// escape sequence -- note that this is protected; it can only be called
//   from another lexer rule -- it will not ever directly return a token to
//   the parser
// There are various ambiguities hushed in this rule.  The optional
// '0'...'9' digit matches should be matched here rather than letting
// them go back to STRING_LITERAL to be matched.  ANTLR does the
// right thing by matching immediately; hence, it's ok to shut off
// the FOLLOW ambig warnings.
protected
ESC
    :   '\\'
        (   'n'
        |   'r'
        |   't'
        |   'b'
        |   'f'
        |   '"' //NOI18N
        |   '\''
        |   '\\'
        |   ('u')+ HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT 
        |   ('0'..'3')
            (
                options {
                    warnWhenFollowAmbig = false;
                }
            :   ('0'..'7')
                (   
                    options {
                        warnWhenFollowAmbig = false;
                    }
                :   '0'..'7'
                )?
            )?
        |   ('4'..'7')
            (
                options {
                    warnWhenFollowAmbig = false;
                }
            :   ('0'..'9')
            )?
        )
    ;

// hexadecimal digit (again, note it's protected!)
protected
HEX_DIGIT
    :   ('0'..'9'|'A'..'F'|'a'..'f')
    ;


// a numeric literal
INT_LITERAL
    {   
        boolean isDecimal=false;
        int tokenType = DOUBLE_LITERAL; 
    }
    :   '.' {_ttype = DOT;}
            (('0'..'9')+ {tokenType = DOUBLE_LITERAL;}
             (EXPONENT)? 
             (tokenType = FLOAT_SUFFIX | DOUBLE_SUFFIX)? 
            { _ttype = tokenType; })?
    |   (   '0' {isDecimal = true;} // special case for just '0'
            (   ('x'|'X')
                (                                           // hex
                    // the 'e'|'E' and float suffix stuff look
                    // like hex digits, hence the (...)+ doesn't
                    // know when to stop: ambig.  ANTLR resolves
                    // it correctly by matching immediately.  It
                    // is therefor ok to hush warning.
                    options {
                        warnWhenFollowAmbig=false;
                    }
                :   HEX_DIGIT
                )+
            |   ('0'..'7')+                                 // octal
            )?
        |   ('1'..'9') ('0'..'9')*  {isDecimal=true;}       // non-zero decimal
        )
        (   ('l'|'L') { _ttype = LONG_LITERAL; }
        
        // only check to see if it's a float if looks like decimal so far
        |   {isDecimal}?
            {tokenType = DOUBLE_LITERAL;} 
            (   '.' ('0'..'9')* (EXPONENT)? 
                (tokenType = FLOAT_SUFFIX | DOUBLE_SUFFIX)?
            |   EXPONENT (tokenType = FLOAT_SUFFIX | DOUBLE_SUFFIX)?
            |   tokenType = FLOAT_SUFFIX
            |   DOUBLE_SUFFIX
            )
            { _ttype = tokenType; }
        )?
    ;

// a couple protected methods to assist in matching floating point numbers
protected
EXPONENT
    :   ('e'|'E') ('+'|'-')? ('0'..'9')+
    ;

//MBO:
// I split FLOAT_SUFFIX into two rules FLOAT_SUFFIX and DOUBLE_SUFFIX. 
// Then antlr-2.7.0 generates explicit case switches for 'f', 'F' and  'd', 'D'
// instead of huge bitsets.

protected
FLOAT_SUFFIX returns [int tokenType]
    : ( 'f' | 'F' ) { tokenType = FLOAT_LITERAL; } 
    ;

protected
DOUBLE_SUFFIX
    : ( 'd' | 'D' )
    ;

// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer

// NOTE, this rule includes an ambiguity regarding Java identifiers (isJavaIdentifierStart).
// ANTLR generates a lot of warnings, you can ignore these warnings. 
// To test the grammar for other "real" ambiguities, comment out the lines that call 
// isJavaIdentifierStart and isJavaIdentifierPart.

// VERY IMPORTANT: IDENT must be the last public rule of the lexer! The ANTLR generated method nextToken 
// calls IDENT for any valid input character, thus public rules added after IDENT are never called.
IDENT
    options {paraphrase = "an identifier"; testLiterals=true;} //NOI18N
    :   (   'a'..'z'
        |   'A'..'Z'
        |   '_'
        |   '$'
        |   UNICODE_STR
        |   { Character.isJavaIdentifierStart(LA(1)) }? .
        ) 
        (   'a'..'z'
        |   'A'..'Z'
        |   '_'
        |   '$'
        |   UNICODE_STR
        |   UNICODE_DIGIT
        |   { Character.isJavaIdentifierPart(LA(1)) }? .
        )*
    ;

protected
UNICODE_DIGIT
    : '0'..'9'
    | '\uff10'..'\uff19'
    ;

protected
UNICODE_STR
    : '\\' ('u')+ HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
        {
            try {
                String tmp = text.toString();
                char c  = (char)Integer.parseInt(tmp.substring(tmp.length() - 4, tmp.length()), 16);
                // problems using ANTLR feature $setText => use generated code
                text.setLength(_begin); 
                text.append(new Character(c).toString());
            }
            catch (NumberFormatException ex) {
                errorMsg.fatal(msg.msg("ERR_UnexpectedExceptionUnicode"), ex); //NOI18N
            }
        }
    ;

//===== Parser Class Definitions =====

/**
 * This class defines the syntax analysis (parser) of the JDOQL compiler.
 *
 * @author  Michael Bouschen
 */
class JDOQLParser extends Parser;

options {
    k = 2;                   // two token lookahead
    exportVocab = JDOQL;
    buildAST = true;
    ASTLabelType = "JDOQLAST"; // NOI18N
}

tokens
{
    // "imaginary" tokens, that have no corresponding real input

    QUERY_TREE             <AST=com.sun.jdori.common.query.tree.Tree>;
    CANDIDATE_CLASS        <AST=com.sun.jdori.common.query.tree.CandidateClass>;
    PARAMETER_DECL         <AST=com.sun.jdori.common.query.tree.Node>;
    VARIABLE_DECL          <AST=com.sun.jdori.common.query.tree.VariableDecl>;

    // operators
    OBJECT_EQUAL           <AST=com.sun.jdori.common.query.tree.EqualsExpr>;
    OBJECT_NOT_EQUAL       <AST=com.sun.jdori.common.query.tree.NotEqualsExpr>;
    COLLECTION_EQUAL       <AST=com.sun.jdori.common.query.tree.EqualsExpr>;
    COLLECTION_NOT_EQUAL   <AST=com.sun.jdori.common.query.tree.NotEqualsExpr>;
    UNARY_MINUS            <AST=com.sun.jdori.common.query.tree.UnaryMinusExpr>;
    UNARY_PLUS             <AST=com.sun.jdori.common.query.tree.UnaryPlusExpr>;
    CONCAT                 <AST=com.sun.jdori.common.query.tree.PlusExpr>;
    CAST                   <AST=com.sun.jdori.common.query.tree.CastExpr>;

    // special dot expressions
    FIELD_ACCESS           <AST=com.sun.jdori.common.query.tree.FieldAccessExpr>;
    STATIC_FIELD_ACCESS    <AST=com.sun.jdori.common.query.tree.FieldAccessExpr>;
    NAVIGATION             <AST=com.sun.jdori.common.query.tree.FieldAccessExpr>;
    CONTAINS               <AST=com.sun.jdori.common.query.tree.ContainsCallExpr>;
    STARTS_WITH            <AST=com.sun.jdori.common.query.tree.StartsWithCallExpr>;
    ENDS_WITH              <AST=com.sun.jdori.common.query.tree.EndsWithCallExpr>;
    IS_EMPTY               <AST=com.sun.jdori.common.query.tree.IsEmptyCallExpr>;
    
    // identifier types
    VARIABLE_ACCESS        <AST=com.sun.jdori.common.query.tree.VariableAccessExpr>;
    PARAMETER_ACCESS       <AST=com.sun.jdori.common.query.tree.ParameterAccessExpr>;

    // types
    TYPE                   <AST=com.sun.jdori.common.query.tree.Type>;

    // constant value
    CONSTANT               <AST=com.sun.jdori.common.query.tree.ConstantExpr>;

    // Query Tree nodes
    BOOLEAN_LITERAL        <AST=com.sun.jdori.common.query.tree.BooleanLiteralExpr>;
    BYTE_LITERAL           <AST=com.sun.jdori.common.query.tree.ByteLiteralExpr>;
    SHORT_LITERAL          <AST=com.sun.jdori.common.query.tree.ShortLiteralExpr>;

    // temporary node removed by Semantic
    ARG_LIST               <AST=com.sun.jdori.common.query.tree.Node>;
}

{
    /** The error message support class. */
    protected ErrorMsg errorMsg;
    
    /** I18N support. */
    protected final static I18NHelper msg = 
        I18NHelper.getInstance("com.sun.jdori.common.query.Bundle"); //NOI18N
    
    /**
     *
     */
    public void init(ErrorMsg errorMsg)
    {
        this.errorMsg = errorMsg;
    }
    
    /**
     * ANTLR method called when an error was detected.
     */
    public void reportError(RecognitionException ex)
    {
        JDOQLLexer.handleANTLRException(ex, errorMsg);
    }

    /**
     * ANTLR method called when an error was detected.
     */
    public void reportError(String s)
    {
        errorMsg.error(0, 0, s);
    }

    /**
     *
     */
    public void reportError(int line, int column, String s)
    {
        errorMsg.error(line, column, s);
    }

    /**
     * ANTLR method called when a warning was detected.
     */
    public void reportWarning(String s)
    {
        throw new JDOQueryException(s);
    }

}

// ----------------------------------
// rules: import declaration
// ----------------------------------

parseImports
{   
    errorMsg.setContext("declareImports");  //NOI18N
}
    :   ( declareImport ( SEMI! declareImport )* )? ( SEMI! )? EOF!
    ;

declareImport
    :   i:IMPORT^ q:qualifiedNameStar
        {
            if (#q.getType() == STAR) {
                #i.setType(IMPORT_ON_DEMAND);
                #i.setFirstChild(#q.getFirstChild());
            }
        }
    ;

// ----------------------------------
// rules: parameter declaration
// ----------------------------------

parseParameters
{   
    errorMsg.setContext("declareParameters"); //NOI18N
}
    :   ( declareParameter ( COMMA! declareParameter )* )? ( COMMA! )? EOF!
    ;

declareParameter
    :   type IDENT
        { 
            ParameterDecl paramDecl = 
                new ParameterDecl(new JDOQLLexer.JDOQLToken(PARAMETER_DECL,"parameterDecl")); //NOI18N
            paramDecl.setFirstChild(#declareParameter);
            #declareParameter = paramDecl;
    }
    ;

// ----------------------------------
// rules: variables declaration
// ----------------------------------

parseVariables
{   
    errorMsg.setContext("declareVariables");  //NOI18N
}
    :   ( declareVariable ( SEMI! declareVariable )* )? ( SEMI! )? EOF!
    ;

declareVariable
    :   type IDENT
        {  
            VariableDecl variableDecl = 
                new VariableDecl(new JDOQLLexer.JDOQLToken(VARIABLE_DECL,"variableDecl")); //NOI18N
            variableDecl.setFirstChild(#declareVariable);
            #declareVariable = variableDecl;
        }
    ;

// ----------------------------------
// rules ordering specification
// ----------------------------------

parseOrdering
{   
    errorMsg.setContext("setOrdering");  //NOI18N
}
    :   ( orderSpec ( COMMA! orderSpec )* )? ( COMMA! )? EOF!
    ;

orderSpec
    :   e:expression ( ASCENDING^ | DESCENDING^ )
    ; 

// ----------------------------------
// rules filer expression
// ----------------------------------

parseFilter
{  
    errorMsg.setContext("setFilter");  //NOI18N
}
    :   e:expression EOF!
    ;

// This is a list of expressions.
expressionList
    :   expression (COMMA! expression)*
    ;

expression
    :   invalidExpression
    ;

// assignment expression: not supported => syntax error
invalidExpression
    :   conditionalOrExpression 
        ( 
            op:invalidOperator
            {
                errorMsg.error(#op.getLine(), #op.getColumn(), 
                    msg.msg("EXC_UnexpectedToken", #op.getText())); //NOI18N
            }
            invalidExpression
        )?
    ;

invalidOperator
    :   ASSIGN | PLUS_ASSIGN | MINUS_ASSIGN | STAR_ASSIGN | 
        DIV_ASSIGN | BAND_ASSIGN | BOR_ASSIGN | SR | SL
    ;

// conditional or ||
conditionalOrExpression
    :   conditionalAndExpression (OR^ conditionalAndExpression)*
    ;

// conditional and &&
conditionalAndExpression
    :   inclusiveOrExpression (AND^ inclusiveOrExpression)*
    ;

// bitwise or logical or |
inclusiveOrExpression
    :   andExpression (BOR^ andExpression)*
    ;

// bitwise or logical and &
andExpression
    :   equalityExpression (BAND^ equalityExpression)*
    ;

// equality/inequality ==/!=
equalityExpression
    :   relationalExpression ((NOT_EQUAL^ | EQUAL^) relationalExpression)*
    ;
// boolean relational expressions
relationalExpression
    :   additiveExpression
        (   (   LT^
            |   GT^
            |   LE^
            |   GE^
            )
            additiveExpression
        )*
    ;

// binary addition/subtraction
additiveExpression
    :   multiplicativeExpression ((PLUS^ | MINUS^) multiplicativeExpression)*
    ;
// multiplication/division/modulo
multiplicativeExpression
    :   unaryExpression ((STAR^ | DIV^ ) unaryExpression)*
    ;

unaryExpression
    :   MINUS^ {#MINUS.setType(UNARY_MINUS);} unaryExpression
    |   PLUS^  {#PLUS.setType(UNARY_PLUS);} unaryExpression
    |   i:INC^ unaryExpression
        {
            errorMsg.error(#i.getLine(), #i.getColumn(), 
                msg.msg("EXC_UnexpectedToken", #i.getText())); //NOI18N
        }
    |   d:DEC^ unaryExpression
        {
            errorMsg.error(#d.getLine(), #d.getColumn(), 
                msg.msg("EXC_UnexpectedToken", #d.getText())); //NOI18N
        }
    |   unaryExpressionNotPlusMinus
    ;

unaryExpressionNotPlusMinus
    :   BNOT^ unaryExpression
    |   LNOT^ unaryExpression
    |   ( LPAREN type RPAREN unaryExpression )=>
          lp:LPAREN^ {#lp.setType(CAST);} type RPAREN! unaryExpression
    |   postfixExpression
    ;

// qualified names, field access, method invocation
postfixExpression
    :   primary
        (   DOT^ IDENT ( argList )? )*
    ;

argList
    :   LPAREN!
        (   expressionList
            {
                Node argListRoot = new Node(new JDOQLLexer.JDOQLToken(ARG_LIST,"argList")); //NOI18N
                argListRoot.setFirstChild(#argList);
                #argList = argListRoot;
            }
        |   /* empty list */
            {
                Node argListRoot = new Node(new JDOQLLexer.JDOQLToken(ARG_LIST,"argList")); //NOI18N
                #argList = argListRoot;
            }
        )
        RPAREN!
    ;

// the basic element of an expression
primary
    :   IDENT
    |   literal
    |   THIS
    |   LPAREN! expression RPAREN!
    ;

literal
    :   TRUE
    |   FALSE
    |   INT_LITERAL
    |   LONG_LITERAL
    |   FLOAT_LITERAL
    |   DOUBLE_LITERAL
    |   c:CHAR_LITERAL
        {
            // strip quotes from the token text
            String text = #c.getText();
            #c.setText(text.substring(1,text.length()-1));
        }
    |   s:STRING_LITERAL
        {
            // strip quotes from the token text
            String text = #s.getText();
            #s.setText(text.substring(1,text.length()-1));
        }
    |   NULL
    ;

qualifiedName
    :   IDENT ( DOT^ IDENT )*
    ;

qualifiedNameStar
    :   IDENT ( DOT^ IDENT )* ( DOT! STAR^ )?
    ;

type
    :   qualifiedName
    |   primitiveType
    ;

// The primitive types.
primitiveType
    :   BOOLEAN
    |   BYTE
    |   CHAR
    |   SHORT
    |   INT
    |   FLOAT
    |   LONG
    |   DOUBLE
    ;
